@@ -23,6 +23,7 @@ import Algorithms.BF.QuantumIf
 import Algorithms.BF.Hex
 import QuipperLib.QFT 
 import QuipperLib.Simulation
+import QuipperLib.Decompose
 
 import Libraries.Auxiliary (mmap)
 
@@ -814,38 +815,38 @@ shift_right (d:d':ds) = do
 -- displayed separately, or in various combinations.
 
 -- | Displays the overall Boolean Formula circuit for a given oracle description.
-main_circuit :: Format -> BooleanFormulaOracle -> IO ()
-main_circuit f oracle = print_simple f (qw_bf oracle)
+main_circuit :: Format -> GateBase -> BooleanFormulaOracle -> IO ()
+main_circuit f g oracle = print_simple f (decompose_generic g (qw_bf oracle))
 
 -- | Display just 1 time-step of the given oracle,
 --   i.e., one iteration of the 'u' from 'exp_u', with no controls.
-main_u :: Format -> BooleanFormulaOracle -> IO ()
-main_u f o = print_generic f (u o) (registerShape o)
+main_u :: Format -> GateBase -> BooleanFormulaOracle -> IO ()
+main_u f g o = print_generic f (decompose_generic g (u o)) (registerShape o)
 
 -- | Display just 1 time-step of the 'walk' algorithm for the given oracle,
 --   i.e., one iteration of /walk/, with no controls.
-main_walk :: Format -> BooleanFormulaOracle -> IO ()
-main_walk f o = print_generic f walk (registerShape o)
+main_walk :: Format -> GateBase -> BooleanFormulaOracle -> IO ()
+main_walk f g o = print_generic f (decompose_generic g walk) (registerShape o)
 
 -- | Display just 1 time-step of the 'diffuse' algorithm for the given oracle,
 --   i.e., one iteration of /diffuse/, with no controls.
-main_diffuse :: Format -> BooleanFormulaOracle -> IO ()
-main_diffuse f o= print_generic f diffuse (registerShape o)
+main_diffuse :: Format -> GateBase -> BooleanFormulaOracle -> IO ()
+main_diffuse f g o = print_generic f (decompose_generic g diffuse) (registerShape o)
 
 -- | Display just 1 time-step of the 'oracle' algorithm for the given oracle,
 --   i.e., one iteration of /oracle/, with no controls.
-main_oracle :: Format -> BooleanFormulaOracle -> IO ()
-main_oracle f o = print_generic f (oracle o) (registerShape o)
+main_oracle :: Format -> GateBase -> BooleanFormulaOracle -> IO ()
+main_oracle f g o = print_generic f (decompose_generic g (oracle o)) (registerShape o)
 
 -- | Display just 1 time-step of the 'undo_oracle' algorithm for the given oracle,
 --   i.e., one iteration of /undo_oracle/, with no controls.
-main_undo_oracle :: Format -> BooleanFormulaOracle -> IO ()
-main_undo_oracle f o = print_generic f (undo_oracle o) (registerShape o)
+main_undo_oracle :: Format -> GateBase -> BooleanFormulaOracle -> IO ()
+main_undo_oracle f g o = print_generic f (decompose_generic g (undo_oracle o)) (registerShape o)
 
 -- | Display the circuit for the Hex algorithm, for the given oracle,
 -- i.e., one iteration of 'hex_oracle', with no controls.
-main_hex :: Format -> BooleanFormulaOracle -> IO ()
-main_hex f o = print_generic f (hex_oracle init s x_max) (pos,binary)
+main_hex :: Format -> GateBase -> BooleanFormulaOracle -> IO ()
+main_hex f g o = print_generic f (decompose_generic g (hex_oracle init s x_max)) (pos,binary)
   where
     init = start_board o
     s = oracle_s o
@@ -856,8 +857,8 @@ main_hex f o = print_generic f (hex_oracle init s x_max) (pos,binary)
 
 -- | Display the circuit for the Checkwin_red algorithm, for the given oracle,
 -- i.e., one iteration of 'checkwin_red_circuit', with no controls.
-main_checkwin_red :: Format -> BooleanFormulaOracle -> IO ()
-main_checkwin_red f o = print_generic f (checkwin_red_circuit x_max) (qshape redboard,qubit)
+main_checkwin_red :: Format -> GateBase -> BooleanFormulaOracle -> IO ()
+main_checkwin_red f g o = print_generic f (decompose_generic g (checkwin_red_circuit x_max)) (qshape redboard,qubit)
   where
     (redboard,_) = start_board o
     x_max = oracle_x_max o  
