@@ -67,32 +67,27 @@ while cond func x = if (cond x) then while cond func (func x) else x
 -- | Like 'while', but with a known bound on number of iterations. This
 --   construct can be converted to a quantum circuit, while a general unbounded
 --   'while' cannot be lifted.
-boundedWhile :: (Integral int) => (a -> Bool) -> int -> (a -> a) -> a -> a
-boundedWhile cond bound func x
-    | bound <  0                   = error "boundedWhile: negative bound"
+bounded_while :: (Integral int) => (a -> Bool) -> int -> (a -> a) -> a -> a
+bounded_while cond bound func x
+    | bound <  0                   = error "bounded_while: negative bound"
     | bound == 0 && (not $ cond x) = x
-    | bound == 0                   = error "boundedWhile: last iteration doesn't satisfy condition"
+    | bound == 0                   = error "bounded_while: last iteration doesn't satisfy condition"
     | otherwise  =
-                   boundedWhile cond (bound-1) func x'
+                   bounded_while cond (bound-1) func x'
                    where x' = if (cond x) then func x else x
 
 -- | A bounded version of Haskell 'iterate' function that produces an infinite
 --   list. This function produces a finite bounded list.
-boundedIterate :: (Integral int) => int -> (a -> a) -> a -> [a]
-boundedIterate bound func x
-    | bound <  0 = error "negative bound in boundedIterate"
+bounded_iterate :: (Integral int) => int -> (a -> a) -> a -> [a]
+bounded_iterate bound func x
+    | bound <  0 = error "negative bound in bounded_iterate"
     | bound == 0 = [x]
     | otherwise  =
-                   x : boundedIterate (bound-1) (func) (func x)
+                   x : bounded_iterate (bound-1) (func) (func x)
 
 -- ===========================================
 -- ** Mathematical functions
 
--- | Generate primes up to a given number. See implementation of 'primes' for
---   details.
-primesTo :: (Integral a) => a -> [a]
-primesTo k =  takeWhile (<= k) primes
-
 -- | Generate primes using the Sieve of Eratosthenes. Straightforward
 --   implementation - when a prime is found, filter all of its multiples
 --   out of the already filtered list. This implementation may eventually blow
@@ -104,28 +99,33 @@ primes  = primesn (2 : [ k*2+1 | k <- [1..] ])
                 n : (primesn (filter (\k -> k `mod` n /= 0) xs))
           primesn [] = undefined -- keep the compiler happy
 
+-- | Generate primes up to a given number. See implementation of 'primes' for
+--   details.
+primes_to :: (Integral a) => a -> [a]
+primes_to k =  takeWhile (<= k) primes
+
 -- | Check if a number is square-free (by brute force).
-isSquareFree :: (Integral a) => a -> Bool
-isSquareFree n = 
+is_square_free :: (Integral a) => a -> Bool
+is_square_free n = 
     (n /= 0) && 
     (not $ any (\p -> (p*p) `divides` n) $ takeWhile (\x -> x^2 <= (abs n)) [2..])
 
 -- | Compute the Jacobi symbol. The definition and algorithm description is
 --   taken from <http://en.wikipedia.org/wiki/Jacobi_symbol>.
-jacobiSymbol :: (Integral a, Num b) => a -> a -> b
-jacobiSymbol a p =
-        jacobiSymbol' a p
+jacobi_symbol :: (Integral a, Num b) => a -> a -> b
+jacobi_symbol a p =
+        jacobi_symbol' a p
     where
         -- The actual implementation of the algorithm. Splitted away to allow
         -- logging.
-        jacobiSymbol' a p
+        jacobi_symbol' a p
             | a == 0        = 0
             | a == 1        = 1                                                       -- Rule 4
             | a == 2        = if (p `mod` 8 == 1 || p `mod` 8 == 7) then 1 else -1    -- Rule 8
-            | a >= p        = jacobiSymbol (a `mod` p) p                              -- Rule 2
-            | 2 `divides` a = (jacobiSymbol (a `div` 2) p) * (jacobiSymbol 2 p)       -- Rule 4
+            | a >= p        = jacobi_symbol (a `mod` p) p                             -- Rule 2
+            | 2 `divides` a = (jacobi_symbol (a `div` 2) p) * (jacobi_symbol 2 p)     -- Rule 4
             | otherwise     = (if p `mod` 4 == 3 && a `mod` 4 == 3 then -1 else 1)    -- Rule 6
-                              * (jacobiSymbol p a)
+                              * (jacobi_symbol p a)
 
 -- | @'mod_with_max' /x/ /y/ /max/@: reduce /x/ modulo /y/, returning the unique representative /x'/ in the range /max/ – /y/ < /x'/ ≤ /max/.
 mod_with_max :: (Integral a) => a -> a -> a -> a
@@ -139,12 +139,13 @@ divchk nom denom =
         then nom `div` denom
         else error ("divchk: " ++ show denom ++ " does not divide " ++ show nom ++ "!")
 
--- | Extended Euclid algorithm ([7] p. 860).
-extendedEuclid :: Integral a => a -> a -> (a, a, a)
-extendedEuclid a b =
+-- | @'extended_euclid' /a/ /b/@: return (/d/,/x/,/y/), 
+-- such that /d/ = gcd(/a/,/b/), and /ax/ + /by/ = /d/.
+extended_euclid :: Integral a => a -> a -> (a, a, a)
+extended_euclid a b =
     if (b == 0) then (a, 1, 0) else (d, x, y)
     where
-        (d', x', y') = extendedEuclid b (a `mod` b)
+        (d', x', y') = extended_euclid b (a `mod` b)
         (d,  x,  y ) = (d', y', x' - (a `div` b)*y')
 
 -- | @/a/ `divides` /b/@: return 'True' if /a/ divides /b/.
@@ -164,14 +165,14 @@ is_int x = x == (fromIntegral $ round x)
 -- 
 -- where /n/ ≥ 0, /a/[sub 0] is an integer, /a/[sub 1], …, /a/[sub /n/] are
 -- positive integers, and /a/[sub /n/] ≠ 1 unless /n/=0. This is called the 
--- (short) continued fraction expansion of /q/. The function 'continuedList' inputs
+-- (short) continued fraction expansion of /q/. The function 'continued_list' inputs
 -- two integers /num/ and /denom/, computes the continued fraction expansion of 
 -- /q/ = /num/ \/ /denom/, and returns the non-empty sequence 
 -- [/a/[sub 0], …, /a/[sub /n/]].  
-continuedList :: (Integral int) => int -> int -> [int]
-continuedList _   0     = []
-continuedList num denom =
-    int_part : continuedList denom num'
+continued_list :: (Integral int) => int -> int -> [int]
+continued_list _   0     = []
+continued_list num denom =
+    int_part : continued_list denom num'
     where
         int_part = num `div` denom
         num'     = num - int_part*denom
@@ -212,58 +213,63 @@ blackbox n out_shape = box n $ \qx -> do
 
 -- $ To reduce the printed sizes of circuits, we box all imported arithmetic components. 
 
+-- | Like 'box', but prepends \"Arith.\" to subroutine names, 
+-- as a crude form of namespace management.
+arithbox :: (QCData qa, QCData qb, QCurry qa_qb qa qb) => String -> qa_qb -> qa_qb
+arithbox n = box ("Arith." ++ n)
+
 -- | Boxed analogue of 'Arith.q_ext_euclid'.
 q_ext_euclid :: QDInt -> QDInt -> Circ (QDInt,QDInt,QDInt,QDInt,QDInt)
-q_ext_euclid = box "q_ext_euclid" Arith.q_ext_euclid
+q_ext_euclid = arithbox "q_ext_euclid" Arith.q_ext_euclid
 
 -- | Boxed analogue of 'Arith.q_add'.
 q_add :: QDInt -> QDInt -> Circ (QDInt,QDInt,QDInt)
-q_add = box "q_add" Arith.q_add
+q_add = arithbox "q_add" Arith.q_add
 
 -- | Boxed analogue of 'Arith.q_mult'.
 q_mult :: (QCData qa, QNum qa)
        => qa -> qa -> Circ (qa,qa,qa)
-q_mult = box "q_mult" Arith.q_mult
+q_mult = arithbox "q_mult" Arith.q_mult
 
 -- | Boxed analogue of 'Arith.q_div_exact'.
 q_div_exact :: QDInt -> QDInt -> Circ (QDInt, QDInt, QDInt)
-q_div_exact = box "q_div_exact" Arith.q_div_exact
+q_div_exact = arithbox "q_div_exact" Arith.q_div_exact
 
 -- | Boxed analogue of 'Arith.q_add_in_place'.
 q_add_in_place :: QDInt -> QDInt -> Circ (QDInt, QDInt)
-q_add_in_place = box "q_add_in_place" Arith.q_add_in_place
+q_add_in_place = arithbox "q_add_in_place" Arith.q_add_in_place
 
 -- | Boxed analogue of 'Arith.q_add_param_in_place'.
 q_add_param_in_place ::IntM -> QDInt -> Circ QDInt
 q_add_param_in_place = Arith.q_add_param_in_place
 {- Currently doesn’t work:
 q_add_param_in_place n = 
-  box ("q_add_param_in_place; n = " ++ show n ++ "; ") (Arith.q_add_param_in_place n)
+  arithbox ("q_add_param_in_place; n = " ++ show n ++ "; ") (Arith.q_add_param_in_place n)
 -}
 
 -- | Boxed analogue of 'Arith.q_div'.
 q_div :: QDInt -> QDInt -> Circ (QDInt, QDInt, QDInt)
-q_div = box "q_div" Arith.q_div
+q_div = arithbox "q_div" Arith.q_div
 
 -- | Boxed analogue of 'Arith.q_mult_param'.
 q_mult_param :: IntM -> QDInt -> Circ (QDInt, QDInt)
 q_mult_param = Arith.q_mult_param
 {- Currently doesn’t work:
 q_mult_param n =
-  box ("q_mult_param; n = " ++ show n ++ "; ") (Arith.q_mult_param n)
+  arithbox ("q_mult_param; n = " ++ show n ++ "; ") (Arith.q_mult_param n)
 -}
 
 -- | Boxed analogue of 'Arith.q_mod_unsigned'.
 q_mod_unsigned :: QDInt -> QDInt -> Circ (QDInt, QDInt, QDInt)
-q_mod_unsigned = box "q_mod_unsigned" Arith.q_mod_unsigned
+q_mod_unsigned = arithbox "q_mod_unsigned" Arith.q_mod_unsigned
 
 -- | Boxed analogue of 'Arith.q_sub_in_place'.
 q_sub_in_place :: QDInt -> QDInt -> Circ (QDInt, QDInt)
-q_sub_in_place = box "q_sub_in_place" Arith.q_sub_in_place
+q_sub_in_place = arithbox "q_sub_in_place" Arith.q_sub_in_place
 
 -- | Boxed analogue of 'Arith.q_increment'.
 q_increment :: QDInt -> Circ QDInt
-q_increment = box "q_increment" Arith.q_increment
+q_increment = arithbox "q_increment" Arith.q_increment
 
 -- ===========================================
 -- ** Other arithmetic functions
@@ -368,14 +374,14 @@ q_mod_2times_buggy x a = do
 -- ** Looping combinators
 
 -- | Perform a bounded while loop, whose body may produce extra output.  
-bounded_while_with_garbage :: (QData qa, QCData qb) =>
+q_bounded_while_with_garbage :: (QData qa, QCData qb) =>
   (qa -> Circ (qa,Qubit))  -- ^ the conditional test on the data
   -> Int                   -- ^ a bound on the number of times the loop can run
   -> qa                    -- ^ the starting value
   -> (qa -> Circ (qa,qb))  -- ^ the body of the loop
   -> Circ (qa,qa)          -- ^ the initial and final values, and the produced data
-bounded_while_with_garbage _ 0 x _ = qc_copy_fun x
-bounded_while_with_garbage test bound x body = do
+q_bounded_while_with_garbage _ 0 x _ = qc_copy_fun x
+q_bounded_while_with_garbage test bound x body = do
   (x, x_really_out) <- with_computed_fun x
     (\x -> do
       x_out <- qinit $ qc_false x 
@@ -402,14 +408,14 @@ bounded_while_with_garbage test bound x body = do
 -- | Perform a bounded-length while loop, with an endo-typed body.
 --
 -- Note: uses /2 * bound/ ancillas.  Can this be avoided?
-bounded_while :: (QCData qa) =>
+q_bounded_while :: (QCData qa) =>
   (qa -> Circ (qa,Qubit))  -- ^ the conditional statement
   -> Int                   -- ^ a bound on the number of times the loop can run
   -> qa                    -- ^ the starting value
   -> (qa -> Circ qa)       -- ^ the body of the loop
   -> Circ (qa,qa)          -- ^ return the initial value, and the final post-loop value
-bounded_while _ 0 x _ = qc_copy_fun x
-bounded_while test bound x body =
+q_bounded_while _ 0 x _ = qc_copy_fun x
+q_bounded_while test bound x body =
   with_computed_fun x
     (\x -> do
       (x,c) <- test x 
@@ -437,15 +443,15 @@ bounded_while test bound x body =
           return ((x,c,c',c''),x'))
 
 -- | Perform a bounded while loop, whose body may produce extra output.  
-bounded_while_productive :: (QCData qa, QCData qb) =>
+q_bounded_while_productive :: (QCData qa, QCData qb) =>
   (qa -> Circ (qa,Qubit))  -- ^ the conditional test on the data
   -> Int                   -- ^ a bound on the number of times the loop can run
   -> qa                    -- ^ the starting value
   -> (qa -> Circ (qa,qb))  -- ^ the body of the loop
   -> Circ (qa,qa,[qb])     -- ^ the initial and final values, and the produced data
-bounded_while_productive test bound x body = do
+q_bounded_while_productive test bound x body = do
   ((x,[]),(x',ys)) <-
-    bounded_while 
+    q_bounded_while 
       (\(x,ys) -> do (x,c) <- test x; return ((x,ys),c))
       bound
       (x,[])
@@ -453,12 +459,12 @@ bounded_while_productive test bound x body = do
   return (x,x',ys)
 
 -- | Perform a bounded “do_until” loop.  
-do_until :: (QCData qa) =>
-  Int                   -- ^ a bound on the number of times the loop can run
+q_do_until :: (QCData qa) =>
+  Int                      -- ^ a bound on the number of times the loop can run
   -> qa                    -- ^ the starting value
   -> (qa -> Circ (qa,Qubit))  -- ^ the body of the loop, producing an input to the next iteration, plus a qubit to mark if we’re finished yet. 
-  -> Circ (qa,qa)     -- ^ return the initial and final values
-do_until bound x body = do
+  -> Circ (qa,qa)          -- ^ return the initial and final values
+q_do_until bound x body = do
   with_computed_fun x body
     (\(x,c) -> do_aux (bound-1) x c)
   where
