@@ -19,7 +19,7 @@ import Algorithms.CL.Auxiliary
 import Algorithms.CL.Types
 import Algorithms.CL.RegulatorClassical
 import Algorithms.CL.RegulatorQuantum
-import Algorithms.CL.SNF
+import Algorithms.CL.SmithReduction
 
 import Data.Ratio
 import Data.Maybe
@@ -27,21 +27,22 @@ import Data.List
 import System.Random
 
 -- ======================================================================
--- * Stage 1 (quantum): Approximate regulator to nearest integer
+-- * Stage 1 (quantum): Approximate regulator to low precision
 -- ======================================================================
 
 -- | Quantum part of the procedure to approximate the regulator /R/.
 --
--- A version of the Hidden Subgroup Problem (HSP) Algorithm is used, 
--- to estimate the (irrational) period of the function /f/ [sub /N/]
--- ('fN', 'q_fN'); this is the function /h/ of [Jozsa, Section 9],
+-- Follows the procedure described in [Jozsa 2003], Sec. 10. An adapted
+-- version of the Hidden Subgroup Problem (HSP) Algorithm is used to
+-- estimate the (irrational) period of the function /f/ [sub /N/]
+-- ('fN', 'q_fN'); this is the function /h/ of [Jozsa 2003], Sec. 9,
 -- discretized with precision /N/ = 2 [super −/n/], and so has weak
--- period /S/ = /NR/.
+-- period /S/ = /NR/.  The precision /n/ is determined by 'n_of_bigD'.
 --
 -- Inputs: Δ; /i/, an assumed bound such that /S/ ≤ 2[sup /i/]; 
 -- and a random “jitter” parameter.
-approximateRegulatorCircuit :: CLIntP -> Int -> CLIntP -> (Circ CInt)
-approximateRegulatorCircuit bigD i jitter = do
+approximate_regulator_circuit :: CLIntP -> Int -> CLIntP -> (Circ CInt)
+approximate_regulator_circuit bigD i jitter = do
     let n = n_of_bigD bigD
         nn = 2^n
         s_bound = 2^i
@@ -77,76 +78,75 @@ approximateRegulatorCircuit bigD i jitter = do
 
 -- | Attempt to approximate the regulator /R/, given an assumed 
 -- bound /i/ such that /S/ ≤ 2[sup /i/], using the probabilistic 
--- quantum computation @approximateRegulatorCircuit@ twice as
--- described in [Jozsa, Section 10].
+-- quantum computation @approximate_regulator_circuit@ twice as
+-- described in [Jozsa 2003], Sec. 10.
 -- 
 -- Check the result for success; if it fails, return 'Nothing'.
 --
 -- (The 'IO' monad is slight overkill here: it is just to make a
--- source of randomness available. A tighter approach could use a
+-- source of randomness available. A tighter approach could use e.g. a
 -- monad transformer such as 'RandT', applied to the 'Circ' monad.)
-tryApproximateRegulator :: CLIntP -> Int -> IO (Maybe CLReal)
-tryApproximateRegulator bigD i = do
+try_approximate_regulator :: CLIntP -> Int -> IO (Maybe CLReal)
+try_approximate_regulator bigD i = do
     let n = n_of_bigD bigD
         q = 2 + 2 * i   -- So 2^q = 4 * (s_bound^2), the first power of 2 assumed to  
         qq = 2^q        -- be above 3 S^2.
 
     c_jitter <- getStdRandom $ randomR (0, 16*qq - 1)
-    c <- run_generic_io (undefined :: Double) $ approximateRegulatorCircuit bigD i c_jitter
+    c <- run_generic_io (undefined :: Double) $ approximate_regulator_circuit bigD i c_jitter
     d_jitter <- getStdRandom $ randomR (0, 16*qq - 1)
-    d <- run_generic_io (undefined :: Double) $ approximateRegulatorCircuit bigD i d_jitter
+    d <- run_generic_io (undefined :: Double) $ approximate_regulator_circuit bigD i d_jitter
 
     -- Find the (possibly empty) list of m's that pass the verification
     -- criteria. The candidate m's are constructed from the convergents of c/d
-    let candidate_ms = filter (verifyPeriodMultiple bigD n) $
+    let candidate_ms = filter (verify_period_multiple bigD n) $
                        map (\c_n -> round $ c_n * (fromIntegral qq) / (fromIntegral c)) $
-                       convergents $ continuedList (fromIntegral c) (fromIntegral d)
+                       convergents $ continued_list (fromIntegral c) (fromIntegral d)
 
     return $ if (null candidate_ms)
              then Nothing
              else (Just (fprealx (-n) (head $ sort $ candidate_ms)))
 
--- | @'verifyPeriodMultiple' Δ /n/ /m/@: 
--- check whether /m/ is within 1 of a multiple of the period /S/ of /f/[sub /N/]
+-- | @'verify_period_multiple' Δ /n/ /m/@: 
+-- check whether /m/ is within 1 of a multiple of the period /S/ of /f/[sub /N/].
 --
 -- Since for any ideal /I/, ρ(ρ(/I/)) is distance > ln 2 from /I/, it suffices 
 -- to check whether the unit ideal is within 4 steps either way of /f/[sub /N/](m).
-verifyPeriodMultiple :: CLIntP -> Int -> CLInt -> Bool
-verifyPeriodMultiple bigD n m =
+verify_period_multiple :: CLIntP -> Int -> CLInt -> Bool
+verify_period_multiple bigD n m =
   let jj = fst $ fN m (2^n) 0 0 bigD
   in (unit_ideal bigD)
-    `elem` ([jj] ++ (take 4 $ iterate rho jj) ++ (take 4 $ iterate rhoInv jj))
+    `elem` ([jj] ++ (take 4 $ iterate rho jj) ++ (take 4 $ iterate rho_inv jj))
 
 -- | Approximate the regulator for a given Δ (/bigD/).
 --
--- Repeatedly run @tryApproximateRegulator@ enough times, with increasing
+-- Repeatedly run @'try_approximate_regulator'@ enough times, with increasing
 -- /i/, that it eventually succeeds with high probability. 
-approximateRegulator :: CLIntP -> IO CLReal
-approximateRegulator bigD =
+approximate_regulator :: CLIntP -> IO CLReal
+approximate_regulator bigD =
     -- Create a (infinite and lazy) list of attempts to approximate the
     -- regulator, assuming S ≤ 2^i for i = 1, 2, …
     -- (For each value of /i/, make (36 * i) attempts.)
     -- Run the attempts until one succeeds; return its answer.
     fmap (fromJust . fromJust) $ sequence_until isJust $ concat
-      [ replicate (36*i) (tryApproximateRegulator bigD i) | i <- [1..] ]
+      [ replicate (36*i) (try_approximate_regulator bigD i) | i <- [1..] ]
 
 -- ======================================================================
 -- * Stage 2 (classical): Compute the regulator more accurately. 
 -- ======================================================================
 
--- | Improve the accuracy of the initial estimate of the regulator /R/, for 
+-- | Improve the precision of the initial estimate of the regulator /R/, for 
 -- a quadratic discriminant Δ.
 --
--- The implementation is mainly based on the proof for Theorem 5
--- appearing on Pages 270-271 in Jozsa's paper (Annals of Physics, 2003)
---
-improveAccuracy :: CLReal -> Integer -> CLReal
-improveAccuracy bigR bigD =
+-- The implementation is essentially based on the proof of Theorem 5 of 
+-- [Jozsa 2003].
+improve_regulator_accuracy :: CLReal -> Integer -> CLReal
+improve_regulator_accuracy bigR bigD =
     fRefine dist idealNew
     where
-        (dist, idealNew) = improveAccuracy' bigD
-        improveAccuracy' :: Integer -> (CLReal, IdDist)
-        improveAccuracy' bigd = refineR (head j') ideals
+        (dist, idealNew) = improve_regulator_accuracy' bigD
+        improve_regulator_accuracy' :: Integer -> (CLReal, IdDist)
+        improve_regulator_accuracy' bigd = refineR (head j') ideals
 
         -- -------------------------------------------------------------------------
         -- Step 3: So far we have computed ideals, I_i , I_i-1 , I_i-2 ,.., I_1.
@@ -155,8 +155,8 @@ improveAccuracy bigR bigD =
         -- continue with I_k-1.  Finish when I_1 is reached.
         -- At this point a reduced Ideal J* is produced to the left of R with
         -- R - delta(J*).  In parallel accumulate distances using fomula
-        -- delta(I,rho(I)) = ln |gamma| for I = Z +gammaZ and Proposition 35
-        -- in Jozsa’s paper.  In the end apply repeatedly rho**2 to J* while
+        -- delta(I,rho(I)) = ln |gamma| for I = Z +gammaZ and Proposition 34
+        -- of [Jozsa 2003].  In the end apply repeatedly rho**2 to J* while
         -- distance does not exeed R.
         -- ------------------------------------------------------------------------
         fRefine :: CLReal -> IdDist -> CLReal
@@ -202,10 +202,10 @@ improveAccuracy bigR bigD =
         -- --------------------------------------------------------------------
         -- The following function constructs a list of ideals I_i, I_i-1, I+i-2,..., I_1,
         -- where delta(I_n) < bigR.
-        -- j' is an Ideal, I_i (or I_N in Jozsa's paper), shown in the last paragraph
-        -- on page 270 in the Jozsa's paper.
-        -- ideals contains the list of ideals so far collected - head corresponds to
-        -- the newest entry
+        -- j' is an Ideal, I_i (or I_N of [Jozsa 2003], in first paragraph of p.24)
+        -- 
+        -- @ideals@ contains the list of ideals so far collected, with the
+        -- the newest entry at the head.
         (j', ideals) =  splitAt 1 $ reverse $ refineR1 i0
 
         refineR1 :: IdDist -> [IdDist]
@@ -236,15 +236,15 @@ improveAccuracy bigR bigD =
 -- factors of (/p/), for primes /p/ ≤ 12(ln Δ)[super 2]. ([Haase and
 -- Maier 2006], Prop. 4.4.)  For each /p/, there are at most two such
 -- prime ideals, and they are easily described.
-computeGenerators :: CLIntP -> [IdealRed]
-computeGenerators bigD =
-    map (reduce . idealFromGenerator) $
-      concat [gens_from_prime p | p <- primesTo primesBound, p >= 3]
+compute_generators :: CLIntP -> [IdealRed]
+compute_generators bigD =
+    map (reduce . ideal_from_generator) $
+      concat [gens_from_prime p | p <- primes_to primes_bound, p >= 3]
     where
-      primesBound = (12*((floor $ log $ fromIntegral bigD)^2))
+      primes_bound = (12*((floor $ log $ fromIntegral bigD)^2))
       d = d_of_bigD bigD
       omega = omega_of_bigD bigD
-      omegaBar = conjugate omega
+      omega_bar = conjugate omega
 
       -- the polynomial with roots omega, omega_bar
       f = if (bigD `mod` 4 == 1)
@@ -253,25 +253,25 @@ computeGenerators bigD =
 
       gens_from_prime 2 = case (d `mod` 8) of
         5 -> []                             
-        1 -> [(2, omega), (2, omegaBar)]    
-        3 -> [(2, AlgNumber (-1) (1/2) bigD)]                   
-        7 -> [(2, AlgNumber (-1) (1/2) bigD)]                   
-        _ -> [(2, AlgNumber 0 (1/2) bigD)]
-      gens_from_prime p = case (d `jacobiSymbol` p) of
+        1 -> [(2, omega), (2, omega_bar)]    
+        3 -> [(2, AlgNum (-1) (1/2) bigD)]                   
+        7 -> [(2, AlgNum (-1) (1/2) bigD)]                   
+        _ -> [(2, AlgNum 0 (1/2) bigD)]
+      gens_from_prime p = case (d `jacobi_symbol` p) of
         0 -> [(p, omega - c)]
-        1 -> [(p, omega - c),(p, c - omegaBar)]
+        1 -> [(p, omega - c),(p, c - omega_bar)]
         _ -> []
         where
           c = fromIntegral $ head [ x | x <- [0..p-1], (f x) `mod` p == 0 ] 
 
       -- give the standard form of the ideal generated by (p,x),
       -- provided that x is of the form (b + √Δ)/2
-      idealFromGenerator :: (CLIntP, AlgNumber) -> Ideal
-      idealFromGenerator (p, w) =
-        let b = (2*w - (AlgNumber 0 1 bigD))
-        in assert (snd_Number b == 0 && is_int (fst_Number b)) 
-          ("computeGenerators: w = " ++ show w ++ " is not of expected form")
-          (Ideal bigD 1 1 (fromIntegral p) (truncate (fst_Number b)))
+      ideal_from_generator :: (CLIntP, AlgNum) -> Ideal
+      ideal_from_generator (p, w) =
+        let b = (2*w - (AlgNum 0 1 bigD))
+        in assert (snd_AlgNum b == 0 && is_int (fst_AlgNum b)) 
+          ("compute_generators: w = " ++ show w ++ " is not of expected form")
+          (Ideal bigD 1 1 (fromIntegral p) (truncate (fst_AlgNum b)))
 
 -- ======================================================================
 -- * Stage 4 (quantum): Find relations between generators.
@@ -292,7 +292,7 @@ hI ideal_I a b j n l =
         ideal_K = (unit_ideal bigD, 0)  
         i       = a                                                  
         (i', ideal_J', ideal_K') =
-            boundedWhile condition bound body (i, ideal_J, ideal_K)  
+            bounded_while condition bound body (i, ideal_J, ideal_K)  
             where
                 bound        = ceiling $ logBase 2 (fromIntegral a)
                 
@@ -307,8 +307,8 @@ hI ideal_I a b j n l =
         ideal_K'' = ideal_K' `star` ideal_J''                        
 
 -- | Compute the ideals from the generators (ĝ function).
-computeGHat :: (Integral int) => [IdDist] -> [int] -> IdDist
-computeGHat gs is =
+compute_ghat :: (Integral int) => [IdDist] -> [int] -> IdDist
+compute_ghat gs is =
     foldl1 (\g_i g_j -> g_i `star` g_j) $
         map (\(ik, gk) -> gk `power` ik) $ zip is gs
 
@@ -324,12 +324,12 @@ compute_i_N_at reg_I reg_out =
     error "incomplete"
 
 
--- | Compute register sizes for @structureCircuit@, given
+-- | Compute register sizes for @structure_circuit@, given
 --  Δ and a precise estimate of /R/. Return a 7-tuple
 --   (/q/,1,2,3,4,5,6) where /q/ is the size of the first
 --   /k/ registers, and 1…6 are the sizes of registers /k/+1…/k/+6.
-registerSizes :: CLIntP -> CLReal -> (Int,Int,Int,Int,Int,Int,Int)
-registerSizes bigD r =
+register_sizes :: CLIntP -> CLReal -> (Int,Int,Int,Int,Int,Int,Int)
+register_sizes bigD r =
     (q, r1, r2, r3, r4, r5, r6)
     where
         q  =     clog2i bigD
@@ -370,7 +370,7 @@ registerSizes bigD r =
                 found = find satisfies convs
 
                 convs = convergents $
-                            continuedList
+                            continued_list
                                 (numerator   $ toRational $ br)
                                 (denominator $ toRational $ br)
 
@@ -390,8 +390,8 @@ registerSizes bigD r =
 
 -- | The quantum circuit used in computing the structure of /CL/(/K/),
 -- given Δ, a precise estimate of /R/, and a generating set for /CL/(/K/).
-structureCircuit :: CLIntP -> CLReal -> [IdealRed] -> (Circ [CInt])
-structureCircuit bigD r gs = do
+structure_circuit :: CLIntP -> CLReal -> [IdealRed] -> (Circ [CInt])
+structure_circuit bigD r gs = do
     -- Apply H to first k registers of size 'q' each
     reg_ks <- qinit [ intm q 0 | x <- gs ]
     reg_ks <- mapUnary hadamard reg_ks
@@ -404,7 +404,7 @@ structureCircuit bigD r gs = do
     reg_i_N <- qinit (intm reg_i_N_size 0)      -- register 4
 
     -- Apply U_g_hat
---  Incomplete: (q_computeGHat gs) reg_ks reg_I
+--  Incomplete: (q_compute_ghat gs) reg_ks reg_I
 
     -- Superposition of distances
     reg_i <- mapUnary hadamard reg_i
@@ -422,7 +422,7 @@ structureCircuit bigD r gs = do
 --  Incomplete: uncompute_i_N_at reg_I reg_i_N
 
     -- Uncompute I
---  Incomplete: q_computeGHat gs reg_ks reg_I
+--  Incomplete: q_compute_ghat gs reg_ks reg_I
 
     -- Measure and discard result (used to project the system)
     result_discard <- measure reg_fIN
@@ -442,12 +442,12 @@ structureCircuit bigD r gs = do
         (q,
          reg_I_size,   reg_i_size,
          reg_fIN_size, reg_i_N_size,
-         _,            _)              = registerSizes bigD r
+         _,            _)              = register_sizes bigD r
 
 -- | Compute the relations between a given set of reduced generators.
-computeRelations :: CLIntP -> CLReal -> [IdealRed] -> IO [CLInt]
-computeRelations bigD r generators = do
-    result <- run_generic_io (undefined :: Double) $ structureCircuit bigD r generators
+compute_relations :: CLIntP -> CLReal -> [IdealRed] -> IO [CLInt]
+compute_relations bigD r generators = do
+    result <- run_generic_io (undefined :: Double) $ structure_circuit bigD r generators
     return (result)
 
 -- ======================================================================
@@ -456,23 +456,23 @@ computeRelations bigD r generators = do
 
 -- | The full implementation of Hallgren’s algorithm.
 -- 
--- @classNumber dd t@: computes the class number |/CL/(/K/)| for Δ = /dd/,
+-- @class_number dd t@: computes the class number |/CL/(/K/)| for Δ = /dd/,
 -- with success probability at least (1 - 1\/2[sup /t/]).
-classNumber :: CLIntP -> Int -> IO(CLInt)
-classNumber bigD t = do
-    -- Stage 1: Compute nearest int to regulator
-    approximateRegulator <- approximateRegulator bigD
+class_number :: CLIntP -> Int -> IO(CLInt)
+class_number bigD t = do
+    -- Stage 1: Approximate regulator to low precision
+    approximate_regulator <- approximate_regulator bigD
 
     -- Stage 2 & 3: Classical algorithms
-    let regulator        =  improveAccuracy approximateRegulator bigD
-        generators       =  computeGenerators    bigD
+    let regulator        =  improve_regulator_accuracy approximate_regulator bigD
+        generators       =  compute_generators bigD
 
         k                = length generators
         q                = ceiling $ logBase 2 $ fromIntegral bigD
         bigT             = t + k * (ceiling $ logBase 10 $ fromIntegral q)
 
     -- Stage 4: Use HSP to compute set of relations
-    relations <- mapM (\_ -> computeRelations bigD regulator generators) [ 1 .. bigT ]
+    relations <- mapM (\_ -> compute_relations bigD regulator generators) [ 1 .. bigT ]
 
     -- Stage 5: Make canonical set and derive CL(K) from structure
-    return (classNumberFromMatrix $ matrixFromList relations)
+    return $ group_order_from_matrix $ matrix_from_list relations
