@@ -33,8 +33,8 @@ unit_idealRed bigD = assert (is_valid_bigD bigD) ("unit_idealRed: " ++ show bigD
 -- | The integer constant /c/ of an ideal.
 --   ([Jozsa 2003], page 14 bottom: \"Since 4/a/ divides /b/[sup 2]-/D/
 --   (cf. proposition 16) we introduce the integer /c/ = |/D/ − /b/[sup 2]|\/(4/a/)\")
-cOfIdeal :: Ideal -> CLInt
-cOfIdeal i@(Ideal bigD m l a b) =
+c_of_ideal :: Ideal -> CLInt
+c_of_ideal i@(Ideal bigD m l a b) =
     if (num `mod` denom == 0) then num `div` denom
                               else error error_string
     where num   = abs ((fromIntegral bigD) - b*b)
@@ -43,12 +43,12 @@ cOfIdeal i@(Ideal bigD m l a b) =
 
 -- | γ(/I/) = (/b/+√Δ)\/(2/a/) for a given ideal /I/.
 --   ([Jozsa 2003], Sec. 6.2.)
-gammaOfIdeal :: Ideal -> AlgNumber
-gammaOfIdeal (Ideal bigD m l a b) = AlgNumber a' b' bigD
+gamma_of_ideal :: Ideal -> AlgNum
+gamma_of_ideal (Ideal bigD m l a b) = AlgNum a' b' bigD
     where
         a' = (fromIntegral b / fromIntegral (2*a)) :: CLRational
         b' = (fromIntegral 1 / fromIntegral (2*a)) :: CLRational
-  -- Recall: @'AlgNumber' u v bigD@ represents (u + v * √bigD).
+  -- Recall: @'AlgNum' u v bigD@ represents (u + v * √bigD).
 
 -- | The reduction function ρ on ideals.
 -- ([Jozsa 2003], Sec. 6.2.)
@@ -60,14 +60,14 @@ rho ii@(Ideal bigD m l a b) = (Ideal bigD m'' l'' a' b')
         l'   = l * a'
         m''  = m' `div` (gcd m' l')
         l''  = l' `div` (gcd m' l')
-        -- From  Jozsa's paper, page 15, formula (10)
-        a'   = cOfIdeal ii
+        -- From  [Jozsa 2003], p.15, equation (10)
+        a'   = c_of_ideal ii
         b'   = tau bigD (-b) a'
 
 -- | The ρ[sup -1] function on ideals.  Inverse to 'rho'.
 -- ([Jozsa 2003], Sec. 6.4.)
-rhoInv :: Ideal -> Ideal
-rhoInv (Ideal bigD m l a b) = (Ideal bigD m'' l'' a'' b'')
+rho_inv :: Ideal -> Ideal
+rho_inv (Ideal bigD m l a b) = (Ideal bigD m'' l'' a'' b'')
     where
         -- Create m and l
         m'    = m * a
@@ -86,33 +86,33 @@ rho_red :: IdealRed -> IdealRed
 rho_red = to_reduced . rho . forget_reduced
 
 -- | The ρ[sup –1] operation on reduced ideals.
-rhoInv_red :: IdealRed -> IdealRed
-rhoInv_red = to_reduced . rhoInv . forget_reduced
+rho_inv_red :: IdealRed -> IdealRed
+rho_inv_red = to_reduced . rho_inv . forget_reduced
 
 -- | The ρ operation on ideals-with-distance.
 rho_d :: IdDist -> IdDist
 rho_d (ii, del) = (rho ii, del + del_change)
   where
-    gamma = gammaOfIdeal ii
-    gamma_bar_by_gamma = (floating_of_Number $ conjugate gamma) / (floating_of_Number gamma)
+    gamma = gamma_of_ideal ii
+    gamma_bar_by_gamma = (floating_of_AlgNum $ conjugate gamma) / (floating_of_AlgNum gamma)
     del_change = (log $ abs gamma_bar_by_gamma) / 2
 
 -- | The ρ[sup –1] operation on ideals-with-distance.
-rhoInv_d :: IdDist -> IdDist
-rhoInv_d (ii, del) = (ii', del - del_change)
+rho_inv_d :: IdDist -> IdDist
+rho_inv_d (ii, del) = (ii', del - del_change)
   where
-    ii' = rhoInv ii
-    gamma = gammaOfIdeal ii'
-    gamma_bar_by_gamma = (floating_of_Number $ conjugate gamma) / (floating_of_Number gamma)
+    ii' = rho_inv ii
+    gamma = gamma_of_ideal ii'
+    gamma_bar_by_gamma = (floating_of_AlgNum $ conjugate gamma) / (floating_of_AlgNum gamma)
     del_change = (log $ abs gamma_bar_by_gamma) / 2
 
--- | The ρ operation on ideals-with-generator (i.e. pairs of an ideal /I/ and an 'AlgNumber' /x/ such that /I/ is the principal ideal (/x/)).
-rho_num :: (Ideal,AlgNumber) -> (Ideal,AlgNumber)
-rho_num (ii, gen) = (rho ii, gen / (gammaOfIdeal ii))
+-- | The ρ operation on ideals-with-generator (i.e. pairs of an ideal /I/ and an 'AlgNum' /x/ such that /I/ is the principal ideal (/x/)).
+rho_num :: (Ideal,AlgNum) -> (Ideal,AlgNum)
+rho_num (ii, gen) = (rho ii, gen / (gamma_of_ideal ii))
 
 -- | Apply ρ to an reduced-ideals-with-generator
-rho_red_num :: (IdealRed,AlgNumber) -> (IdealRed,AlgNumber)
-rho_red_num (ii, gen) = (rho_red ii, gen / (gammaOfIdeal $ forget_reduced ii))
+rho_red_num :: (IdealRed,AlgNum) -> (IdealRed,AlgNum)
+rho_red_num (ii, gen) = (rho_red ii, gen / (gamma_of_ideal $ forget_reduced ii))
 
 -- ======================================================================
 -- * Ideal reductions (bounded)
@@ -123,9 +123,9 @@ reduce ii = to_reduced $ while (not . is_reduced) rho ii
 
 -- | Reduce an ideal within a bounded loop. Applies the ρ function
 --   until the ideal is reduced. Used in 'star' and 'fJN' algorithms.
-boundedReduce :: IdDist -> IdDist
-boundedReduce k@(ideal@(Ideal bigD m l a b),dist) = 
-    fst $ boundedWhile condition bound func (k,0)
+bounded_reduce :: IdDist -> IdDist
+bounded_reduce k@(ideal@(Ideal bigD m l a b),dist) = 
+    fst $ bounded_while condition bound func (k,0)
     where
         -- NOTE: some uncertainty regarding loop bound.
         bound             = ceiling $ bound_log + 1
@@ -138,18 +138,18 @@ boundedReduce k@(ideal@(Ideal bigD m l a b),dist) =
 
 -- | Apply a function (like ρ,ρ[sup -1],ρ[sup 2]) to an ideal, bounded
 --   by 3*ln(Δ)\/2*ln(2). Execute while satisfies condition function.
-boundedStep :: (IdDist -> Bool) -> (IdDist -> IdDist) -> IdDist -> IdDist
-boundedStep condition step_function ideal =
+bounded_step :: (IdDist -> Bool) -> (IdDist -> IdDist) -> IdDist -> IdDist
+bounded_step condition step_function ideal =
     -- Execute a bounded while loop
-    boundedWhile condition bound step_function ideal
+    bounded_while condition bound step_function ideal
     where
         bound = ceiling $ (3 * (log $ fromIntegral $ bigD_of_Ideal $ fst ideal)) / (2 * log 2)
 
--- | Like 'boundedStep', but the condition is checked against delta of the
+-- | Like 'bounded_step', but the condition is checked against delta of the
 --   current ideal.
-boundedStepDelta :: (CLReal -> Bool) -> (IdDist -> IdDist) -> IdDist -> IdDist
-boundedStepDelta condition step_function ideal =
-    boundedStep new_condition step_function ideal
+bounded_step_delta :: (CLReal -> Bool) -> (IdDist -> IdDist) -> IdDist -> IdDist
+bounded_step_delta condition step_function ideal =
+    bounded_step new_condition step_function ideal
     where new_condition = (\k -> condition (delta k))
 
 -- ======================================================================
@@ -169,8 +169,8 @@ dot i1@(Ideal bigD1 m1 l1 a1 b1, delta1) i2@(Ideal bigD2 m2 l2 a2 b2, delta2) =
     where
         sqrtd :: CLReal
         sqrtd = sqrt (fromIntegral bigD1)
-        (k', u', v') = extendedEuclid a1 a2
-        (k,  x,  w ) = extendedEuclid k' ((b1 + b2) `divchk` 2)
+        (k', u', v') = extended_euclid a1 a2
+        (k,  x,  w ) = extended_euclid k' ((b1 + b2) `divchk` 2)
         a3 = (a1 * a2) `divchk` (k * k)
         t1 = x * u' * a1 * b2
         t2 = x * v' * a2 * b1
@@ -189,7 +189,7 @@ dot' i1@(Ideal bigD1 m1 l1 a1 b1, delta1) =
     where
         sqrtd :: CLReal
         sqrtd = sqrt (fromIntegral bigD1)
-        (k, u, w) = extendedEuclid (abs a1) (abs b1)
+        (k, u, w) = extended_euclid (abs a1) (abs b1)
         a3 = (a1 * a1) `divchk` (k * k)
         t1 = u * a1 * b1
         t3 = w*(b1 * b1 + (fromIntegral bigD1)) `divchk` 2
@@ -209,13 +209,13 @@ dot' i1@(Ideal bigD1 m1 l1 a1 b1, delta1) =
 star :: IdDist -> IdDist -> IdDist
 star i j =
     if (delta k1 <= delta i_dot_j)
-        then       boundedStepDelta (<= delta i_dot_j) rho_d k1
-        else rho_d $ boundedStepDelta (>= delta i_dot_j) rhoInv_d k1
+        then       bounded_step_delta (<= delta i_dot_j) rho_d k1
+        else rho_d $ bounded_step_delta (>= delta i_dot_j) rho_inv_d k1
     where
         i_dot_j = i `dot` j
         -- FIX: Over bound (infinite loop) in reducing the following:
         --      <m:1 l:3 a:3 b:2 bigD:28 del:1.1>*<m:1 l:3 a:3 b:4 bigD:28 del:1.6>
-        k1      = boundedReduce i_dot_j
+        k1      = bounded_reduce i_dot_j
 
 -- ======================================================================
 -- * The function f[sub /N/], and variants
@@ -247,7 +247,7 @@ fN_d i j nn ll bigD =
 
         -- Generate Jk's (make an infinite list and take only what is needed)
         jks      = takeWhile (\jk -> (delta jk) <= injl) $
-                        boundedIterate bound_jks
+                        bounded_iterate bound_jks
                             (\jk -> jk `star` jk) j1
                    where
                        -- Bound for jk generation
@@ -266,11 +266,11 @@ fN_d i j nn ll bigD =
             if (delta (jstar `star` jk) <= injl) then jstar `star` jk else jstar
 
         -- Go forward one step at a time as much as needed
-        j_star_17   = boundedStepDelta (< injl) (rho_d.rho_d) j_star_14
+        j_star_17   = bounded_step_delta (< injl) (rho_d.rho_d) j_star_14
 
         -- Go back one step if needed
-        j_star_19   = if (delta (rhoInv_d j_star_17) >= injl)
-                      then rhoInv_d j_star_17
+        j_star_19   = if (delta (rho_inv_d j_star_17) >= injl)
+                      then rho_inv_d j_star_17
                       else j_star_17
 
 -- | Analogue of 'fN', working within the cycle determined by a given ideal /J/.
@@ -290,13 +290,13 @@ fJN_d ideal_J i j nn ll bigD =
 
         -- Generate I and K
         ideal_I = fst (fN_d i j nn ll bigD)
-        ideal_K = boundedReduce (ideal_I `dot` ideal_J)
+        ideal_K = bounded_reduce (ideal_I `dot` ideal_J)
 
         -- Step forward/backward as much as needed
         ideal_KFinal =
             if (delta ideal_K <= injl)
-            then        boundedStepDelta (<  injl) (rho_d)    ideal_K
-            else rho_d $ boundedStepDelta (>= injl) (rhoInv_d) ideal_K
+            then        bounded_step_delta (<  injl) (rho_d)    ideal_K
+            else rho_d $ bounded_step_delta (>= injl) (rho_inv_d) ideal_K
 
 -- ======================================================================
 -- * Classical period-finding
@@ -351,7 +351,7 @@ regulator bigD = snd $ head $ dropWhile (\(ii,_) -> ii /= calO) $ tail $ iterate
 -- by finding (classically) the order of ρ.
 --
 -- Uses '(Ideal,Number)' and 'rho_num'.
-fundamental_unit :: CLIntP -> AlgNumber
+fundamental_unit :: CLIntP -> AlgNum
 fundamental_unit bigD = maximum [eps, -eps, 1/eps, -1/eps]
   where eps = snd $ first_return_with_projection fst rho_num (calO,1)
         calO = unit_ideal bigD
@@ -370,7 +370,7 @@ fundamental_solution d = head pell_solutions
         eps0 = fundamental_unit bigD
         pell_solutions = 
           [ (round x, round y) | n <- [1..],
-                                 let eps@(AlgNumber a b _) = eps0^n,
+                                 let eps@(AlgNum a b _) = eps0^n,
                                  a >= 0, b >= 0,
                                  let x = a,
                                  let y = if bigD == d then b else 2*b,
@@ -397,7 +397,7 @@ regulator_fixed_prec b bigD = fmap snd (first_return_with_proj_bdd b' fst rho_d
         n = n_of_bigD bigD
   -- S = 2RN, so this /i/ gives an assumed bound on log_2 S:  
         i = 1 + b + n
-  -- Following precisions are as used in 'approximateRegulatorCircuit', 'q_fN':
+  -- Following precisions are as used in 'approximate_regulator_circuit', 'q_fN':
         q = 2 + 2 * i 
         l = 4 + q + n
         p = precision_for_fN bigD n l
@@ -407,15 +407,15 @@ regulator_fixed_prec b bigD = fmap snd (first_return_with_proj_bdd b' fst rho_d
 
 
 {-
-Waiting for 'AlgNumber' to be rebased using 'IntM' instead of 'Integer'.
+Waiting for 'AlgNum' to be rebased using 'IntM' instead of 'Integer'.
 
 -- | Find the fundamental unit ε[sub 0] of a field, given the discriminant Δ, 
 -- by finding (classically) the order of ρ,
 -- using fixed-precision arithmetic: 'fix_sizes_Ideal' for 'Ideal's,
--- and a given /l/ for the 'AlgNumber's generating them.
+-- and a given /l/ for the 'AlgNum's generating them.
 --
 -- Uses '(Ideal,Number)' and 'rho_num'.
-fundamental_unit_with_fixed :: Int -> CLIntP -> AlgNumber
+fundamental_unit_with_fixed :: Int -> CLIntP -> AlgNum
 fundamental_unit_with_fixed l bigD = maximum [eps, -eps, 1/eps, -1/eps]
   where eps = snd $ head $ dropWhile (\(ii,_) -> ii /= calO) $ tail $ iterate rho_num $ (calO,one)
         calO = fix_sizes_Ideal $ unit_ideal bigD
