@@ -10,7 +10,7 @@
 --
 -- The key operation for stage 1 is 'q_fN', implementing /f/[sub /N/], the quasi-periodic
 -- function used in approximating the regulator.  This is the function /h/ of
--- [Jozsa 2003, Section 9], discretized with precision 1\//N/ and translated by a
+-- [Jozsa 2003], Sec. 9, discretized with precision 1\//N/ and translated by a
 -- specified jitter parameter.
 --
 -- The key functions for stage 4 are not yet implemented.  These essentially
@@ -313,7 +313,7 @@ q_star_prod = box "q_star_prod" $ \iid jjd -> do
       label (kkd) ("1/ka (II.JJ)")
 
       comment "q_star_prod, step 2: while K not reduced, set K <- rho(K)"
-      (kkd_initial,kkd_reduced) <- bounded_while_with_garbage
+      (kkd_initial,kkd_reduced) <- q_bounded_while_with_garbage
         (\(kk,dist) -> do
           comment "q_star_prod, step 2, loop conditional: is K reduced yet?"
           (kk,c) <- q_is_reduced kk
@@ -346,7 +346,7 @@ q_star_prod = box "q_star_prod" $ \iid jjd -> do
           kkd = (kk,dist_k)
 
       comment "q_star_prod, step 3a: pull-forwards loop"
-      ((dist_ij, kkd), (dij', kkd_forwards)) <- bounded_while_with_garbage
+      ((dist_ij, kkd), (dij', kkd_forwards)) <- q_bounded_while_with_garbage
         (\(dist_ij,(kk,dist_k)) -> do
           comment "q_star_prod, step 3a, loop condition: is δ(K) ≤ δ(II.JJ) still?"
           (dist_k, dist_ij, k_lt_ij) <- q_lt dist_k dist_ij
@@ -359,7 +359,7 @@ q_star_prod = box "q_star_prod" $ \iid jjd -> do
           return ((dij,kkd_new),kkd_old))
 
       comment "q_star_prod, step 3b: pull-backwards loop"
-      ((dist_ij, kkd), (dij'', kkd_too_far_back)) <- bounded_while_with_garbage
+      ((dist_ij, kkd), (dij'', kkd_too_far_back)) <- q_bounded_while_with_garbage
         (\(dist_ij,(kk,dist_k)) -> do
           comment "q_star_prod, step 3b, loop condition: is δ(K) ≥ δ(II.JJ) still?"
           (dist_k, dist_ij, k_gt_ij) <- q_gt dist_k dist_ij
@@ -397,7 +397,7 @@ q_star_square = \iid -> with_computed_fun iid qc_copy_fun
 
 -- |  @'q_fN' Δ /s/ /n/ /l/ /qi/ /j/@: find the minimal ideal-with-distance (/J/,δ[sub /J/]) such that δ[sub /J/] > /x/, where /x/ = /i/\//N/ + /j/\//L/, where /N/ = 2[sup /n/], /L/ = 2[sup /l/].  /qi/ is quantum; other inputs are classical parameters.  Return (/i/,/J/,δ[sub /J/]–/x/).  Work under the assumption that /R/ < 2[sup /s/].
 --
--- This is the function /h/ of [Jozsa 2003, Section 9], discretized with precision 1\//N/ = 2[sup −/n/],
+-- This is the function /h/ of [Jozsa 2003], Sec. 9, discretized with precision 1\//N/ = 2[sup −/n/],
 -- and perturbed by the jitter parameter /j/\//L/.
 q_fN :: CLIntP -> Int -> Int -> QDInt -> IntM -> Circ (QDInt,(IdealRedQ,FPRealQ))
 q_fN bigD n l qi j = do
@@ -468,7 +468,7 @@ q_fN bigD n l qi j = do
       comment "q_fN: (J_*,δ_*) is now the greatest ideal/distance pair ≤ x constructible from the iterated star-squares (J_k,δ_k)."
 
       comment "q_fN, step 5: apply ρ^2 to (J_*,δ_*) as long as δ_* ≤ x."
-      ((jjd_star_initial, qx), (jjd_star, qx_copy)) <- bounded_while_with_garbage
+      ((jjd_star_initial, qx), (jjd_star, qx_copy)) <- q_bounded_while_with_garbage
         -- conditional test:
         (\((jj_star, dist_star), qx) -> do
           (dist_star, qx, test) <- q_le dist_star qx
