@@ -72,90 +72,93 @@ d_of_bigD bigD = case (bigD `mod` 4) of
   0 -> bigD `div` 4
   _ -> bigD
 
--- | Check if /d/ correctly defines a quadratic number field.
---
--- Query: [Jozsa 2003] simply gives the condition as “positive, square-free”.  But >1 seems to also be assumed; possibly >2 or >3?  For now we allow 1,2,3.
+-- | Check if /d/ is a valid input to Hallgren’s algorithm,
+-- i.e. correctly defines a real quadratic number field.
 is_valid_d :: (Integral a) => a -> Bool
-is_valid_d d = d > 0 && isSquareFree d
+is_valid_d d = d > 1 && is_square_free d
 
--- | Check if Δ is a valid discriminant.  Note: see query on 'is_valid_d'.
+-- | Check if /Δ/ is a valid input to Hallgren’s algorithm,
+-- i.e. is the discriminant of a real quadratic number field.
+-- (Cf. <http://en.wikipedia.org/wiki/Fundamental_discriminant>)
 is_valid_bigD :: (Integral a) => a -> Bool
-is_valid_bigD bigD = bigD > 0 && case (bigD `mod` 4) of
-  1 -> isSquareFree bigD
-  0 -> (d `mod` 4 == 2 || d `mod` 4 == 3) && isSquareFree d
+is_valid_bigD bigD = bigD > 1 && case (bigD `mod` 4) of
+  1 -> is_square_free bigD
+  0 -> (d `mod` 4 == 2 || d `mod` 4 == 3) && is_square_free d
     where d = bigD `div` 4
   _ -> False
 
--- | The (infinite, lazy) list of all valid (i.e. positive, square-free) integers /d/, starting from 5.
+-- | The (infinite, lazy) list of all valid inputs /d/, 
+-- i.e. of all square-free integers above 2.
 all_small_ds :: (Integral int) => [int]
-all_small_ds = filter (\n -> is_valid_d n)  [5..]
+all_small_ds = filter (\n -> is_valid_d n)  [2..]
 
--- | The (infinite, lazy) list of all valid discriminants Δ, apart from the dubious 1, 8, 12 (see note at 'is_valid_d').
+-- | The (infinite, lazy) list of all valid inputs Δ, 
+-- i.e. of all discriminants of real quadratic number fields.
 all_bigDs :: (Integral int) => [int]
 all_bigDs = map bigD_of_d all_small_ds
 
 -- ===========================================
 -- ** Field elements
 
--- | A data type describing a number in the algebraic number field K = ℚ[√Δ]: @'AlgNumber' /a/ /b/ Δ@ represents /a/ + /b/√Δ.
+-- | A data type describing a number in the algebraic number field K = ℚ[√Δ]: @'AlgNum' /a/ /b/ Δ@ represents /a/ + /b/√Δ.
 --
 -- In general, the type of coefficients may be any type of (classical or quantum)
 -- numbers, i.e. an instance of the 'Num' or 'QNum' class.
 -- Given this, the algebraic numbers with a fixed Δ will in turn be an instance
 -- of 'Num' or 'QNum'.
 -- 
--- A value @/a/ :: /x/@ may also be used as an @'AlgNumberGen' /x/@,
+-- A value @/a/ :: /x/@ may also be used as an @'AlgNumGen' /x/@,
 -- with no Δ specified, to represent simply /a/ + 0√Δ; this can be considered polymorphic
 -- over all possible values of Δ. 
 -- 
 -- This is similar to the use of 'IntM's or 'FPReal's of indeterminate size, although
 -- unlike for them, we do not restrict this to the classical case.  However, the
--- question of whether an 'AlgNumberQ' has specified √Δ is (like e.g. the length of
+-- question of whether an 'AlgNumQ' has specified √Δ is (like e.g. the length of
 -- a list) is a parameter property, known at circuit generation time, not a purely 
 -- quantum property.
-data AlgNumberGen a = AlgNumber a a CLIntP | AlgNumber_indet a deriving (Show)
+data AlgNumGen a = AlgNum a a CLIntP | AlgNum_indet a deriving (Show)
 
--- | The specific instance of 'AlgNumberGen' used for classical (parameter) computation.
-type AlgNumber = AlgNumberGen CLRational
+-- | The specific instance of 'AlgNumGen' used for classical (parameter) computation.
+type AlgNum = AlgNumGen CLRational
 
--- | Extract the first co-ordinate of an 'AlgNumberGen'
-fst_Number :: AlgNumberGen a -> a
-fst_Number (AlgNumber u _ _) = u
-fst_Number (AlgNumber_indet u) = u
+-- | Extract the first co-ordinate of an 'AlgNumGen'
+fst_AlgNum :: AlgNumGen a -> a
+fst_AlgNum (AlgNum u _ _) = u
+fst_AlgNum (AlgNum_indet u) = u
 
--- | Extract the second co-ordinate of an 'AlgNumberGen'
-snd_Number :: (Num a) => AlgNumberGen a -> a
-snd_Number (AlgNumber _ v _) = v
-snd_Number (AlgNumber_indet _) = 0
+-- | Extract the second co-ordinate of an 'AlgNumGen'
+snd_AlgNum :: (Num a) => AlgNumGen a -> a
+snd_AlgNum (AlgNum _ v _) = v
+snd_AlgNum (AlgNum_indet _) = 0
 
-instance (Eq a, Num a) => Eq (AlgNumberGen a) where
-  (AlgNumber a b bigD) == (AlgNumber a' b' bigD') = 
+instance (Eq a, Num a) => Eq (AlgNumGen a) where
+  (AlgNum a b bigD) == (AlgNum a' b' bigD') = 
     if bigD == bigD' then a == a' && b == b'
-    else error "Operation = on AlgNumber: operands must have same Δ."
-  (AlgNumber a b bigD) == (AlgNumber_indet a') = (AlgNumber a b bigD) == (AlgNumber a' 0 bigD)
-  (AlgNumber_indet a) == (AlgNumber a' b' bigD') = (AlgNumber a 0 bigD') == (AlgNumber a' b' bigD')
-  (AlgNumber_indet a) == (AlgNumber_indet a') = a == a'
+    else error "Operation = on AlgNum: operands must have same Δ."
+  (AlgNum a b bigD) == (AlgNum_indet a') = (AlgNum a b bigD) == (AlgNum a' 0 bigD)
+  (AlgNum_indet a) == (AlgNum a' b' bigD') = (AlgNum a 0 bigD') == (AlgNum a' b' bigD')
+  (AlgNum_indet a) == (AlgNum_indet a') = a == a'
 
 -- | Print a 'Number' in human-readable (though not Haskell-readable) format, as e.g. 
-pretty_show_Number :: Show a => AlgNumberGen a -> String
-pretty_show_Number (AlgNumber a b bigD) = (show a) ++ " + " ++ (show b) ++ " √" ++ show bigD
-pretty_show_Number (AlgNumber_indet a) = show a
+pretty_show_AlgNum :: Show a => AlgNumGen a -> String
+pretty_show_AlgNum (AlgNum a b bigD) = (show a) ++ " + " ++ (show b) ++ " √" ++ show bigD
+pretty_show_AlgNum (AlgNum_indet a) = show a
 
 -- | Realize an algebraic number as a real number (of any 'Floating' type).
-floating_of_Number :: (Real a, Floating b) => AlgNumberGen a -> b
-floating_of_Number (AlgNumber a b bigD) = (realToFrac a) + (realToFrac b) * (sqrt $ fromIntegral bigD)
-floating_of_Number (AlgNumber_indet a) = (realToFrac a)
+floating_of_AlgNum :: (Real a, Floating b) => AlgNumGen a -> b
+floating_of_AlgNum (AlgNum a b bigD) = (realToFrac a) + (realToFrac b) * (sqrt $ fromIntegral bigD)
+floating_of_AlgNum (AlgNum_indet a) = (realToFrac a)
 
 -- | Coerce one algebraic number into the field of a second, if possible.  If not possible (i.e. if their Δ’s mismatch), throw an error.
-number_promote :: Num a => AlgNumberGen a -> AlgNumberGen b -> ErrMsg -> AlgNumberGen a
-number_promote (AlgNumber a b bigD) (AlgNumber _ _ bigD') e =
-  if bigD == bigD' then AlgNumber a b bigD
+number_promote :: Num a => AlgNumGen a -> AlgNumGen b -> ErrMsg -> AlgNumGen a
+number_promote (AlgNum a b bigD) (AlgNum _ _ bigD') e =
+  if bigD == bigD' then AlgNum a b bigD
   else error $ e "mismatched Δ."
-number_promote (AlgNumber_indet a) (AlgNumber _ _ bigD') _ = AlgNumber a 0 bigD'
-number_promote n (AlgNumber_indet _) _ = n
+number_promote (AlgNum_indet a) (AlgNum _ _ bigD') _ = AlgNum a 0 bigD'
+number_promote n (AlgNum_indet _) _ = n
 
-instance (Ord a, Num a) => Ord (AlgNumberGen a) where
-  compare (AlgNumber a b bigD) (AlgNumber a' b' bigD') = 
+instance (Ord a, Num a) => Ord (AlgNumGen a) where
+  compare (AlgNum a b bigD) (AlgNum a' b' bigD') = 
     if bigD == bigD' then
       case (compare a a', compare b b') of 
         (EQ,y) -> y
@@ -165,83 +168,83 @@ instance (Ord a, Num a) => Ord (AlgNumberGen a) where
         (GT,LT) -> compare ((a-a')^2) ((b-b')^2 * fromInteger bigD)
         (LT,GT) -> compare ((b-b')^2 * fromInteger bigD) ((a-a')^2)
     else 
-      error "compare // AlgNumberGen: mismatched Δ."
-  compare (AlgNumber a b bigD) (AlgNumber_indet a') = compare (AlgNumber a b bigD) (AlgNumber a' 0 bigD)
-  compare (AlgNumber_indet a) (AlgNumber a' b' bigD') = compare (AlgNumber a 0 bigD') (AlgNumber a' b' bigD')
-  compare (AlgNumber_indet a) (AlgNumber_indet a') = compare a a'
+      error "compare // AlgNumGen: mismatched Δ."
+  compare (AlgNum a b bigD) (AlgNum_indet a') = compare (AlgNum a b bigD) (AlgNum a' 0 bigD)
+  compare (AlgNum_indet a) (AlgNum a' b' bigD') = compare (AlgNum a 0 bigD') (AlgNum a' b' bigD')
+  compare (AlgNum_indet a) (AlgNum_indet a') = compare a a'
 
-instance (Ord a, Num a) => Num (AlgNumberGen a) where
-  (AlgNumber a b bigD) + (AlgNumber a' b' bigD') = 
-    if bigD == bigD' then AlgNumber (a+a') (b+b') bigD
-    else error "Operation + on AlgNumber: operands must have same Δ."
-  (AlgNumber a b bigD) + (AlgNumber_indet a') = (AlgNumber a b bigD) + (AlgNumber a' 0 bigD)
-  (AlgNumber_indet a) + (AlgNumber a' b' bigD') = (AlgNumber a 0 bigD') + (AlgNumber a' b' bigD')
-  (AlgNumber_indet a) + (AlgNumber_indet a') = (AlgNumber_indet (a + a'))
+instance (Ord a, Num a) => Num (AlgNumGen a) where
+  (AlgNum a b bigD) + (AlgNum a' b' bigD') = 
+    if bigD == bigD' then AlgNum (a+a') (b+b') bigD
+    else error "Operation + on AlgNum: operands must have same Δ."
+  (AlgNum a b bigD) + (AlgNum_indet a') = (AlgNum a b bigD) + (AlgNum a' 0 bigD)
+  (AlgNum_indet a) + (AlgNum a' b' bigD') = (AlgNum a 0 bigD') + (AlgNum a' b' bigD')
+  (AlgNum_indet a) + (AlgNum_indet a') = (AlgNum_indet (a + a'))
 
-  (AlgNumber a b bigD) * (AlgNumber a' b' bigD') = 
-    if bigD == bigD' then AlgNumber (a*a' + b*b'*(fromIntegral bigD)) (a*b' + a'*b) bigD
-    else error "Operation * on AlgNumber: operands must have same Δ."
-  (AlgNumber a b bigD) * (AlgNumber_indet a') = (AlgNumber a b bigD) * (AlgNumber a' 0 bigD)
-  (AlgNumber_indet a) * (AlgNumber a' b' bigD') = (AlgNumber a 0 bigD') * (AlgNumber a' b' bigD')
-  (AlgNumber_indet a) * (AlgNumber_indet a') = (AlgNumber_indet (a * a'))
+  (AlgNum a b bigD) * (AlgNum a' b' bigD') = 
+    if bigD == bigD' then AlgNum (a*a' + b*b'*(fromIntegral bigD)) (a*b' + a'*b) bigD
+    else error "Operation * on AlgNum: operands must have same Δ."
+  (AlgNum a b bigD) * (AlgNum_indet a') = (AlgNum a b bigD) * (AlgNum a' 0 bigD)
+  (AlgNum_indet a) * (AlgNum a' b' bigD') = (AlgNum a 0 bigD') * (AlgNum a' b' bigD')
+  (AlgNum_indet a) * (AlgNum_indet a') = (AlgNum_indet (a * a'))
 
-  (AlgNumber a b bigD) - (AlgNumber a' b' bigD') = 
-    if bigD == bigD' then AlgNumber (a-a') (b-b') bigD
-    else error "Operation - on AlgNumber: operands must have same Δ."
-  (AlgNumber a b bigD) - (AlgNumber_indet a') = (AlgNumber a b bigD) - (AlgNumber a' 0 bigD)
-  (AlgNumber_indet a) - (AlgNumber a' b' bigD') = (AlgNumber a 0 bigD') - (AlgNumber a' b' bigD')
-  (AlgNumber_indet a) - (AlgNumber_indet a') = (AlgNumber_indet (a - a'))
+  (AlgNum a b bigD) - (AlgNum a' b' bigD') = 
+    if bigD == bigD' then AlgNum (a-a') (b-b') bigD
+    else error "Operation - on AlgNum: operands must have same Δ."
+  (AlgNum a b bigD) - (AlgNum_indet a') = (AlgNum a b bigD) - (AlgNum a' 0 bigD)
+  (AlgNum_indet a) - (AlgNum a' b' bigD') = (AlgNum a 0 bigD') - (AlgNum a' b' bigD')
+  (AlgNum_indet a) - (AlgNum_indet a') = (AlgNum_indet (a - a'))
 
   abs n = if (n >= 0) then n else -n 
   signum n = number_promote (if n > 0 then 1 else if n == 0 then 0 else (-1)) n 
-               (const "CL.Types: internal error (signum // AlgNumber)")
-  fromInteger = AlgNumber_indet . fromInteger
+               (const "CL.Types: internal error (signum // AlgNum)")
+  fromInteger = AlgNum_indet . fromInteger
 
-instance (Real a) => Real (AlgNumberGen a) where
-  toRational = toRational . floating_of_Number
+instance (Real a) => Real (AlgNumGen a) where
+  toRational = toRational . floating_of_AlgNum
 
-instance (Ord a, Fractional a) => Fractional (AlgNumberGen a) where
-  fromRational = AlgNumber_indet . fromRational
+instance (Ord a, Fractional a) => Fractional (AlgNumGen a) where
+  fromRational = AlgNum_indet . fromRational
 
-  recip (AlgNumber a b bigD) = 
+  recip (AlgNum a b bigD) = 
     let c = (a^2) - (b^2 * (fromIntegral bigD))
     in assert (c /= 0) (if (a == 0 && b == 0) then "CL.Types: divide-by-zero error"
-                        else if is_valid_bigD bigD then "CL.Types: internal error (AlgNumber // recip)"
+                        else if is_valid_bigD bigD then "CL.Types: internal error (AlgNum // recip)"
                         else error "CL.Types: " ++ show bigD ++ " not a valid discriminant")
-       (AlgNumber (a/c) (-b/c) bigD)
-  recip (AlgNumber_indet a) = AlgNumber_indet $ recip a
+       (AlgNum (a/c) (-b/c) bigD)
+  recip (AlgNum_indet a) = AlgNum_indet $ recip a
 
-instance (RealFrac a) => RealFrac (AlgNumberGen a) where
+instance (RealFrac a) => RealFrac (AlgNumGen a) where
   properFraction x = (x',x - fromIntegral x')
-    where x' = truncate $ floating_of_Number x
+    where x' = truncate $ floating_of_AlgNum x
 
 -- | The algebraic conjugate: sends /a/ + /b/ √Δ to /a/ - /b/ √Δ.
-conjugate :: (Num a) => AlgNumberGen a -> AlgNumberGen a
-conjugate (AlgNumber a b bigD) = (AlgNumber a (-b) bigD)
-conjugate (AlgNumber_indet a) = (AlgNumber_indet a)
+conjugate :: (Num a) => AlgNumGen a -> AlgNumGen a
+conjugate (AlgNum a b bigD) = (AlgNum a (-b) bigD)
+conjugate (AlgNum_indet a) = (AlgNum_indet a)
 
 -- | Test whether an algebraic number is an algebraic integer.
 --
 -- (A number is an algebraic integer iff it can be written in the form /m/ + /n/(Δ + √Δ)\/2, where /m/, /n/ are integers.
--- See Jozsa, Prop. 14.)
-is_alg_int :: (Ord a, RealFrac a) => AlgNumberGen a -> Bool
-is_alg_int (AlgNumber a b bigD) = is_int n && is_int m
+-- See [Jozsa 2003], proof of Prop. 14.)
+is_alg_int :: (Ord a, RealFrac a) => AlgNumGen a -> Bool
+is_alg_int (AlgNum a b bigD) = is_int n && is_int m
   where
 -- solve for m, n in the equation [a + b √D = m + n(Δ + √Δ)/2]
     n = 2 * b
     m = a - b * fromIntegral bigD
-is_alg_int (AlgNumber_indet a) = is_int a
+is_alg_int (AlgNum_indet a) = is_int a
 
 -- | Test whether an algebraic number is a unit of the ring of algebraic integers.
-is_unit :: (Ord a, RealFrac a) => AlgNumberGen a -> Bool
+is_unit :: (Ord a, RealFrac a) => AlgNumGen a -> Bool
 is_unit n = if n == 0 then False else (is_alg_int n) && (is_alg_int (recip n))
 
 -- | The number ω associated to the field /K/.
-omega_of_bigD :: CLIntP -> AlgNumber
+omega_of_bigD :: CLIntP -> AlgNum
 omega_of_bigD bigD =
   if (bigD `mod` 4 == 1)
-  then (AlgNumber (1/2) (1/2) bigD)
-  else (AlgNumber 0 1 bigD)
+  then (AlgNum (1/2) (1/2) bigD)
+  else (AlgNum 0 1 bigD)
 
 -- ===========================================
 -- * Ideals
@@ -478,12 +481,12 @@ is_standard (Ideal bigD m l a b) =
   && ((bigD - (fromIntegral b)^2) `mod` (4 * (fromIntegral a)) == 0)
   && b == tau bigD b a
 
--- | Test whether an 'Ideal' is reduced.  (An ideal \</m/,/l/,/a/,/b/> is reduced iff /m/ = 1, /l/ = /a/, /b/ ≥ 0 and /b/ + √Δ > 2/a/ (Jozsa, Prop. 20)). 
+-- | Test whether an 'Ideal' is reduced.  (An ideal \</m/,/l/,/a/,/b/> is reduced iff /m/ = 1, /l/ = /a/, /b/ ≥ 0 and /b/ + √Δ > 2/a/ ([Jozsa 2003], Prop. 20)). 
 is_reduced :: Ideal -> Bool
 is_reduced (Ideal bigD m l a b) = (m == 1) && (l == a) && (b >= 0) && (b + root_bigD > 2 * a)
   where root_bigD = ceiling $ sqrt $ fromIntegral bigD 
 
--- | Test whether an 'IdealRed' is really reduced.  (An ideal \<1,/a/,/a/,/b/> is reduced iff /b/ ≥ 0 and /b/ + √Δ > 2/a/ (Jozsa, Prop. 20)). 
+-- | Test whether an 'IdealRed' is really reduced.  (An ideal \<1,/a/,/a/,/b/> is reduced iff /b/ ≥ 0 and /b/ + √Δ > 2/a/ ([Jozsa 2003], Prop. 20)). 
 is_really_reduced :: IdealRed -> Bool
 is_really_reduced (IdealRed bigD a b) = (b >= 0) && (b + root_bigD > 2 * a)
   where root_bigD = ceiling $ sqrt $ fromIntegral bigD 
@@ -492,7 +495,7 @@ is_really_reduced (IdealRed bigD a b) = (b >= 0) && (b + root_bigD > 2 * a)
 forget_reduced :: IdealRed -> Ideal
 forget_reduced (IdealRed bigD a b) = (Ideal bigD 1 a a b)
 
--- | Coerce an 'Ideal' to an 'IdealRed', if it is reduced, or throw an error otherwise.  Cf. Jozsa, Prop. 20.
+-- | Coerce an 'Ideal' to an 'IdealRed', if it is reduced, or throw an error otherwise.  Cf. [Jozsa 2003], Prop. 20.
 to_reduced :: Ideal -> IdealRed
 to_reduced ii@(Ideal bigD m l a b) =
   if is_reduced ii then (IdealRed bigD a b)
@@ -524,7 +527,7 @@ q_tau bigD = box ("tau, Δ = " ++ show bigD) $ \a b -> do
     qc_copy
   return (a,b,t)
 
--- | Test whether a given 'IdealQ' is reduced.  \</m/,/l/,/a/,/b/> is reduced iff /m/ = 1, /l/ = /a/, /b/ ≥ 0 and /b/ + √Δ > 2/a/ (Jozsa, Prop. 20).  
+-- | Test whether a given 'IdealQ' is reduced.  \</m/,/l/,/a/,/b/> is reduced iff /m/ = 1, /l/ = /a/, /b/ ≥ 0 and /b/ + √Δ > 2/a/ ([Jozsa 2003], Prop. 20).  
 q_is_reduced :: IdealQ -> Circ (IdealQ,Qubit) 
 q_is_reduced = box "is_reduced" $ \qii ->
   let bigD = bigD_of_Ideal qii in
@@ -543,7 +546,7 @@ q_is_reduced = box "is_reduced" $ \qii ->
       test_out <- qnot test_out `controlled` tests
       return ((tests, rest), test_out))
 
--- | Test whether a given 'IdealQ' is really reduced (as it should always be, if code is written correctly).  An ideal \<1,/a/,/a/,/b/> is reduced iff /b/ ≥ 0 and /b/ + √Δ > 2/a/ (Jozsa, Prop. 20). 
+-- | Test whether a given 'IdealQ' is really reduced (as it should always be, if code is written correctly).  An ideal \<1,/a/,/a/,/b/> is reduced iff /b/ ≥ 0 and /b/ + √Δ > 2/a/ ([Jozsa 2003], Prop. 20). 
 q_is_really_reduced :: IdealRedQ -> Circ (IdealRedQ,Qubit) 
 q_is_really_reduced = box "is_really_reduced" $ \qii ->
   let bigD = bigD_of_IdealRed qii in
@@ -577,7 +580,7 @@ q_assert_reduced = box "assert_reduced" $ \x@(Ideal bigD m l a b) -> do
 
 -- | Throw a (quantum-runtime) error if an 'IdealRedQ' is not really reduced; otherwise, do nothing.
 --
--- Compare 'assert_reduced', 'q_is_really_reduced' in "Algorithms.CL.RegulatorQuantum", and Jozsa Prop. 20.
+-- Compare 'assert_reduced', 'q_is_really_reduced' in "Algorithms.CL.RegulatorQuantum", and [Jozsa 2003] Prop. 20.
 q_assert_really_reduced :: IdealRedQ -> Circ IdealRedQ
 q_assert_really_reduced = box "assert_really_reduced" $ \ii -> do 
   (ii,test) <- q_is_really_reduced ii
@@ -597,20 +600,24 @@ length_for_ab bigD = 3 + (ceiling $ logBase 2 $ fromIntegral bigD)
 
 -- | Given Δ, return the size of integers to be used for the coefficients /m/, /l/ of general ideals.
 --
--- TODO: bound this!  Neither Hallgren nor Jozsa discusses bounds on the values of /m/ and /l/ that will appear, and we do not yet have a bound.  For now we use the same length as for /a/ and /b/, for convenience; this should be considered a dummy bound, quite possibly not sufficient in general.
+-- TODO: bound this!  Neither Hallgren nor [Jozsa 2003] discusses bounds on the values of /m/ and /l/ that will appear, and we do not yet have a bound.  For now we use the same length as for /a/ and /b/, for convenience; this should be considered a dummy bound, quite possibly not sufficient in general.
 length_for_ml :: CLIntP -> Int
 length_for_ml = length_for_ab
 
--- | Given Δ, return the precision /n/ = log[sub 2]/N/ for the input\/output of /f/[sub /N/].
-n_of_bigD :: (Integral int) => CLIntP -> int
-n_of_bigD bigD = ceiling $ logBase 2 min_dist_recip
-  where min_dist_recip = (32 * (fromIntegral bigD) * (log $ fromIntegral bigD)) / 3
-
--- | Given Δ, /n/, /l/, as 'fN', 'q_fN', return the precision required for intermediate distance calculations during the computation of /f/[sub /N/].
+-- | Given Δ, return the precision /n/ = log[sub 2]/N/ to be used for
+-- discretizing the quasi-periodic function /f/ to /f/[sub /N/].
 --
--- (“Precision” here is given as the number of binary digits after the point).
+-- (“Precision” here means the number of binary digits after the point).
 -- 
--- TODO: bound this more carefully.  The Jozsa paper asks for the final output to be precision /n/, but does not discuss intermediate precision, and we have not yet got a confident answer.  For now, just a back-of-the-envelope estimate, which should be sufficient and /O/(correct), but is almost certainly rather larger than necessary.
+-- Taken to ensure 1\//N/ < 3/(32 Δ log Δ).  (Cf. [Jozsa 2003], Prop. 36 (iii).)
+n_of_bigD :: (Integral int) => CLIntP -> int
+n_of_bigD bigD =
+  ceiling $ logBase 2 $ 32 * (fromIntegral bigD) * (log $ fromIntegral bigD) / 3
+
+-- | Given Δ, /n/, /l/ (as for 'fN', 'q_fN'), return the precision required
+-- for intermediate distance calculations during the computation of /f/[sub /N/].
+--
+-- TODO: bound this more carefully.  [Jozsa 2003] asks for the final output to be precision /n/, but does not discuss intermediate precision, and we have not yet got a confident answer.  For now, just a back-of-the-envelope estimate, which should be sufficient and /O/(correct), but is almost certainly rather larger than necessary.
 precision_for_fN :: CLIntP -> Int -> Int -> Int
 precision_for_fN _ n l = 2 * (n + l)
 
@@ -627,51 +634,3 @@ fix_sizes_IdealRed (IdealRed bigD a b) = (IdealRed bigD (f a) (f b))
   where
     f x = intm_promote x (intm n 0) "set_sizes_Ideal: lengths already fixed incompatibly"
     n = max (length_for_ml bigD) (length_for_ab bigD)
-
--- ===========================================
--- ** Sample data
-
--- $ Some fairly arbitrarily chosen sample elements of these types, for convenience in testing circuit functions.
-
--- | A sample 'CLReal'.
-sample_CLReal :: Int -> FPReal
-sample_CLReal l = (fprealx 0 (intm l 0))
-
--- | A sample 'Ideal'.
-sample_Ideal :: CLIntP -> Ideal
-sample_Ideal bigD =
-  let l = max (length_for_ab bigD) (length_for_ml bigD)
-      x = (intm l 0)
-  in (Ideal bigD x x x x)
-
--- | A sample 'IdealQ'.
-sample_IdealQ :: CLIntP -> IdealQ
-sample_IdealQ = qshape . sample_Ideal
-
--- | A sample 'IdealRed'.
-sample_IdealRed :: CLIntP -> IdealRed
-sample_IdealRed bigD =
-  let l = max (length_for_ab bigD) (length_for_ml bigD)
-      x = (intm l 0)
-  in (IdealRed bigD x x)
-
--- | A sample 'IdealRedQ'.
-sample_IdealRedQ :: CLIntP -> IdealRedQ
-sample_IdealRedQ = qshape . sample_IdealRed
-
--- | A sample 'IdDist'.
-sample_IdDist :: CLIntP -> IdDist
-sample_IdDist bigD = (sample_Ideal bigD, sample_CLReal (length_for_ab bigD))
-
--- | A sample 'IdDistQ'.
-sample_IdDistQ :: CLIntP -> IdDistQ
-sample_IdDistQ = qshape . sample_IdDist
-
-
--- | A sample 'IdRedDist'.
-sample_IdRedDist :: CLIntP -> IdRedDist
-sample_IdRedDist bigD = (sample_IdealRed bigD, sample_CLReal (length_for_ab bigD))
-
--- | A sample 'IdRedDistQ'.
-sample_IdRedDistQ :: CLIntP -> IdRedDistQ
-sample_IdRedDistQ = qshape . sample_IdRedDist
