@@ -112,16 +112,20 @@ import qualified Data.IntMap as IntMap
 --
 -- The multiplier, for 4-bit integers mod 15.
 --
--- * Options and parameters:
+-- @./tf --help@
+--
+-- Print detailed usage info (accepted options, etc.).
+--
+-- * Parameters:
 -- 
--- /l/ is the length of integers used in the oracle.  
--- (Default value: 31.)
+-- /l/: the length of integers used in the oracle.  
+-- (Default value: 4.)
 --
--- /n/ is the size of nodes in the graph.  
--- (Default value: 15.)
+-- /n/: the size of nodes in the graph.  
+-- (Default value: 3.)
 --
--- /r/ is log[sub 2] of the tuple size of the Hamming graph.  
--- (Default value: 9.)
+-- /r/: log[sub 2] of the tuple size of the Hamming graph.  
+-- (Default value: 2.)
 
 --
 -- * Option processing
@@ -212,7 +216,7 @@ options =
   [ 
 -- Generic options
     Option ['h'] ["help"]       (NoArg help)                       "print usage info and exit",
-    Option ['f'] ["format"]     (ReqArg format "<format>")         "output format for circuits (default: eps)",
+    Option ['f'] ["format"]     (ReqArg format "<format>")         "output format for circuits (default: preview)",
     Option ['g'] ["gatebase"]   (ReqArg gatebase "<gatebase>")     "type of gates to decompose into (default: logical)",
 -- Triangle finding parameters
     Option ['l'] ["l"]          (ReqArg lll "<l>")                 "parameter l (default: 4)",
