@@ -45,6 +45,11 @@ import Data.Bits
 
 -- | An enumeration type to represent symbolic basic gates (/X/, /Y/,
 -- /Z/, /H/, /S/, /T/, /W/, /E/).
+-- 
+-- Note: when we use a list of 'Gate's to express a sequence of
+-- operators, the operators are meant to be applied right-to-left,
+-- i.e., as in the mathematical notation for matrix multiplication.
+-- This is the opposite of the quantum circuit notation.
 data Gate = X | Y | Z | H | S | T | E | W
           deriving (Show, Eq)
 
@@ -125,9 +130,6 @@ convert = from_gates . to_gates
 -- ----------------------------------------------------------------------
 -- ** Matrices in /U/(2)
 
--- | A convenient abbreviation for the type of 2×2-matrices.
-type U2 a = Matrix Two Two a
-
 -- | The Pauli /X/ operator.
 u2_X :: (Ring a) => U2 a
 u2_X = matrix2x2 (0, 1)
@@ -189,9 +191,6 @@ instance (RootHalfRing a, ComplexRing a) => FromGates (U2 a) where
 -- $ This is the Bloch sphere representation of single qubit
 -- operators.
 
--- | A convenient abbreviation for the type of 3×3-matrices.
-type SO3 a = Matrix Three Three a
-
 -- | The Pauli /X/ operator.
 so3_X :: (Ring a) => SO3 a
 so3_X = matrix3x3 (1,  0,  0)
@@ -399,10 +398,16 @@ instance FromGates NormalForm where
 -- an element of /SO/(3), i.e., orthogonal with determinant 1.
 --
 -- This implementation uses the Matsumoto-Amano algorithm.
+-- 
+-- Note: the list of gates will be returned in right-to-left order,
+-- i.e., as in the mathematical notation for matrix multiplication.
+-- This is the opposite of the quantum circuit notation.
 synthesis_bloch :: SO3 DReal -> [Gate]
 synthesis_bloch m = aux m1 k
   where
     (m1, k) = denomexp_decompose m
+
+    aux :: SO3 DInteger -> Integer -> [Gate]    
     aux m 0 = to_gates (clifford_of_so3 m)
     aux m k = to_gates axis ++ [T] ++ aux m4 (k-1)
       where
@@ -461,6 +466,10 @@ instance ToGates TwoLevel where
 --
 -- We use a variant of the Kliuchnikov-Maslov-Mosca algorithm, as
 -- implemented in "Libraries.Synthesis.MultiQubitSynthesis".
+-- 
+-- Note: the list of gates will be returned in right-to-left order,
+-- i.e., as in the mathematical notation for matrix multiplication.
+-- This is the opposite of the quantum circuit notation.
 synthesis_u2 :: U2 DOmega -> [Gate]
 synthesis_u2 = to_gates . normalize . synthesis_nqubit
 
