@@ -521,6 +521,12 @@ matrix_controlled (m :: Matrix n n a) = oplus (1 :: Matrix n n a) m
 -- ----------------------------------------------------------------------
 -- ** Constructors and destructors
 
+-- | A convenient abbreviation for the type of 2×2-matrices.
+type U2 a = Matrix Two Two a
+
+-- | A convenient abbreviation for the type of 3×3-matrices.
+type SO3 a = Matrix Three Three a
+
 -- | A convenience constructor for matrices: turn a list of columns
 -- into a matrix. 
 -- 
