@@ -75,6 +75,7 @@ power mul unit = aux where
 -- for exact types such as 'Rational' and 'EReal'.
 floorlog :: (Fractional b, Ord b) => b -> b -> (Integer, b)
 floorlog b x 
+    | x <= 0            = error "floorlog: argument not positive"    
     | 1 <= x && x < b   = (0, x)
     | 1 <= x*b && x < 1 = (-1, b*x)
     | r < b             = (2*n, r)
@@ -451,3 +452,12 @@ newsynth_stats prec theta g = dynamic_fixedprec2 digits f prec theta where
   digits = ceiling (10 + 2 * prec * logBase 10 2)
   f prec theta = (m, err, ct) where
     ((m, err), ct) = keeptrying_count (newsynth_step prec theta) g
+
+-- | A version of 'newsynth' that returns a list of gates instead of a
+-- matrix. The inputs are the same as for 'newsynth'.
+-- 
+-- Note: the list of gates will be returned in right-to-left order,
+-- i.e., as in the mathematical notation for matrix multiplication.
+-- This is the opposite of the quantum circuit notation.
+newsynth_gates :: (RandomGen g) => Double -> SymReal -> g -> [Gate]
+newsynth_gates prec theta g = synthesis_u2 (newsynth prec theta g)
