@@ -21,5 +21,5 @@ import QuipperLib.QuipperASCIIParser
 -- | Main function: read from 'stdin' and call the previewer. 
 main :: IO ()
 main = do
-  (ins,circuit) <- parse_from_stdin
+  (ins,circuit) <- parse_from_stdin_with_handler
   print_generic Preview circuit ins
