@@ -7,6 +7,12 @@
 -- | Functions to decompose circuits into various gate bases.
 
 module QuipperLib.Decompose ( 
+  -- * Precision
+  Precision,
+  bits,
+  digits,
+  -- * Phase
+  KeepPhase,
   -- * Gate bases
   GateBase (..),
   gatebase_enum,
@@ -17,4 +23,4 @@ module QuipperLib.Decompose (
 import QuipperLib.Decompose.CliffordT
 import QuipperLib.Decompose.GateBase
 import QuipperLib.Decompose.Legacy
-
+import QuipperLib.Synthesis
