@@ -5,6 +5,7 @@
 -- ======================================================================
 
 {-# LANGUAGE FlexibleContexts #-}
+{-# LANGUAGE Rank2Types #-}
 
 -- | This module provides transformers for decomposing circuits into
 -- the Clifford+/T/ gate base.
@@ -27,14 +28,26 @@
 -- not-gates (with positive or negative control). Rotation and phase
 -- gates are left unchanged.
 -- 
--- * [bold Strict Clifford+/T/ decomposition.] This decomposes all
--- gates that remain after applying both approximate and exact
--- Clifford+/T/ decomposition into the gate set consisting of just
--- four gates /H/, /S/, /T/, and /CNOT/. If the transformer encounters
--- gates that are not single-qubit Clifford gates, /T/, /T/[sup †], or
--- singly-controlled not-gates (with positive or negative control),
--- then the output is semantically correct, but not in the right gate
--- base.
+-- * [bold Standard gate set.] We define the standard gate set to
+-- consist of the gates /X/, /Y/, /Z/, /H/, /S/, /S/[sup †], /T/,
+-- /T/[sup †], and /CNOT/. This transformer decomposes all gates that
+-- remain after applying both approximate and exact Clifford+/T/
+-- decomposition into the standard gate set. If the transformer
+-- encounters gates that are not single-qubit Clifford gates, /T/,
+-- /T/[sup †], or singly-controlled not-gates (with positive or
+-- negative control), then the output is still semantically correct,
+-- but may not be in the standard gate set.  This transformer
+-- suppresses global phases.
+-- 
+-- * [bold Strict gate set.] We define the strict gate set to consist
+-- of the gates /H/, /S/, /T/, and /CNOT/. This transformer decomposes
+-- all gates that remain after applying both approximate and exact
+-- Clifford+/T/ decomposition into the strict gate set. If the
+-- transformer encounters gates that are not single-qubit Clifford
+-- gates, /T/, /T/[sup †], or singly-controlled not-gates (with
+-- positive or negative control), then the output is still
+-- semantically correct, but may not be in the strict gate set. This
+-- transformer suppresses global phases.
 -- 
 -- These above transformers may be applied in any order. Control
 -- trimming is primarily for demonstration purposes; it does not need
@@ -42,7 +55,9 @@
 -- trimming. The exact and approximate Clifford+/T/ decompositions can
 -- be applied in either order; since they each target a different set
 -- of gates, they must both be performed to obtain a complete
--- decomposition into the Clifford+/T/ gate set.
+-- decomposition into the Clifford+/T/ gate set. The standard and
+-- strict transformers assume that their input has already been
+-- pre-processed by the exact and approximate transformers.
 
 module QuipperLib.Decompose.CliffordT where
 
@@ -244,28 +259,31 @@ trimcontrols_transformer gate@(T_Comment _ _ _) = identity_transformer gate
 
 -- | This transformer decomposes rotation and phase gates into the
 -- Clifford+/T/ basis, using the approximate synthesis algorithm of
--- <http://arxiv.org/abs/1212.6253>.  It requires a precision
--- parameter, as well as a source of randomness. Other gates
--- are unchanged.
-approx_ct_transformer :: (RandomGen g) => Precision -> g -> Transformer Circ Qubit Bit
-approx_ct_transformer prec g gate@(T_GPhase t ncf f) = f $
+-- <http://arxiv.org/abs/1212.6253>. Other gates
+-- are unchanged. 
+-- 
+-- This transformer requires a precision parameter, as well as a
+-- source of randomness. The 'KeepPhase' flag indicates whether to
+-- respect global phases.
+approx_ct_transformer :: (RandomGen g) => KeepPhase -> Precision -> g -> Transformer Circ Qubit Bit
+approx_ct_transformer kp prec g gate@(T_GPhase t ncf f) = f $
   \qs cs -> without_controls_if ncf $ do
     comment_with_label (printf "ENTER: GPhase (t=%f)" t) cs "c"
     with_combined_controls_iX 1 cs $ \qcs -> do
       case qcs of
-        [] -> approximate_synthesis_phase prec theta g
+        [] -> approximate_synthesis_phase kp prec theta g
           where
             theta = pi * to_real t
         [c] -> do
           with_signed_qubit c $ \q -> do
-            approximate_synthesis_phase_ctrl prec theta g q
+            approximate_synthesis_phase_ctrl kp prec theta g q
             return ()
           where
             theta = pi * to_real t
         _ -> error ("approx_ct_transformer: internal error. " ++ show gate)
       comment_with_label "EXIT: GPhase" cs "c"
       return cs
-approx_ct_transformer prec g gate@(T_QRot name _ _ inv t ncf f) = f $
+approx_ct_transformer kp prec g gate@(T_QRot name _ _ inv t ncf f) = f $
   \qs gctls cs -> without_controls_if ncf $ do
     comment_with_label (printf "ENTER: %s%s (t=%f)" name (optional inv "*") t) (qs,cs) ("q","c")
     case (name, inv, qs, gctls) of
@@ -289,13 +307,13 @@ approx_ct_transformer prec g gate@(T_QRot name _ _ inv t ncf f) = f $
           with_combined_controls_iX 1 cs $ \qcs -> do
             case qcs of
               [] -> do
-                approximate_synthesis_phase prec theta g
+                approximate_synthesis_phase kp prec theta g
                 return ()
                 where
                   theta = negate_if inv (to_real (pi*t))
               [c] -> do
                 with_signed_qubit c $ \q2 -> do
-                  approximate_synthesis_phase_ctrl prec theta g q2
+                  approximate_synthesis_phase_ctrl kp prec theta g q2
                   return ()
                 where
                   theta = negate_if inv (to_real (pi*t))
@@ -304,9 +322,14 @@ approx_ct_transformer prec g gate@(T_QRot name _ _ inv t ncf f) = f $
         ("R(2pi/%)", inv, [q], []) -> do
           with_combined_controls_iX 1 (Signed (Endpoint_Qubit q) True : cs) $ \qcs -> do
             case qcs of
+              [] -> do
+                approximate_synthesis_phase kp prec theta g
+                return ()
+                where
+                  theta = negate_if inv (2 * pi / to_real t)
               [c] -> do
                 with_signed_qubit c $ \q1 -> do
-                  approximate_synthesis_phase_ctrl prec theta g q1
+                  approximate_synthesis_phase_ctrl kp prec theta g q1
                   return ()
                 where
                   theta = negate_if inv (2 * pi / to_real t)
@@ -315,9 +338,14 @@ approx_ct_transformer prec g gate@(T_QRot name _ _ inv t ncf f) = f $
         ("T(%)", inv, [q], []) -> do
           with_combined_controls_iX 1 (Signed (Endpoint_Qubit q) True : cs) $ \qcs -> do
             case qcs of
+              [] -> do
+                approximate_synthesis_phase kp prec theta g
+                return ()
+                where
+                  theta = negate_if inv (-to_real t)
               [c] -> do
                 with_signed_qubit c $ \q1 -> do
-                  approximate_synthesis_phase_ctrl prec theta g q1
+                  approximate_synthesis_phase_ctrl kp prec theta g q1
                   return ()
                 where
                   theta = negate_if inv (-to_real t)
@@ -327,13 +355,13 @@ approx_ct_transformer prec g gate@(T_QRot name _ _ inv t ncf f) = f $
           with_combined_controls_iX 1 cs $ \qcs -> do
             case qcs of
               [] -> do
-                approximate_synthesis_phase prec theta g
+                approximate_synthesis_phase kp prec theta g
                 return ()
                 where
                   theta = negate_if inv (-to_real t)
               [c] -> do
                 with_signed_qubit c $ \q2 -> do
-                  approximate_synthesis_phase_ctrl prec theta g q2
+                  approximate_synthesis_phase_ctrl kp prec theta g q2
                   return ()
                 where
                   theta = negate_if inv (-to_real t)
@@ -358,7 +386,7 @@ approx_ct_transformer prec g gate@(T_QRot name _ _ inv t ncf f) = f $
         _ -> error ("approx_ct_transformer: unimplemented gate: " ++ show gate)
     comment_with_label (printf "EXIT: %s%s (t=%f)" name (optional inv "*") t) (qs,cs) ("q","c")
     return (qs, gctls, cs)
-approx_ct_transformer prec g gate = identity_transformer gate
+approx_ct_transformer kp prec g gate = identity_transformer gate
 
 -- ----------------------------------------------------------------------
 -- ** Exact Clifford+/T/ decomposition
@@ -492,6 +520,11 @@ exact_ct_transformer gate@(T_QGate name _ _ inv ncf f) = f $
         ("E", [q], [], []) -> do
           reverse_imp_if inv gate_E_at q
           
+        ("YY", [q], [], [c]) -> do
+          reverse_imp_if inv controlled_YY_at q c
+        ("YY", [q], [], []) -> do
+          reverse_imp_if inv (named_gate_at "YY") q
+          
         ("omega", [q], [], [c]) -> do
           with_signed_qubit c $ \c1 -> do
             reverse_imp_if inv gate_T_at c1
@@ -524,9 +557,15 @@ exact_ct_transformer gate@(T_Comment _ _ _) = identity_transformer gate
 
 
 -- ----------------------------------------------------------------------
--- ** Strict Clifford+/T/ decomposition
+-- ** Decomposition into standard gate set
 
--- | This transformer accepts the following gates:
+-- | This transformer decomposes a circuit into the standard gate set,
+-- which we define to be:
+-- 
+-- * /X/, /Y/, /Z/, /H/, /S/, /S/[sup †], /T/, /T/[sup †], and /CNOT/.
+-- 
+-- As a precondition, the input circuit must only contain the
+-- following gates:
 -- 
 -- * controlled-not (with one positive or negative control),
 -- 
@@ -534,17 +573,141 @@ exact_ct_transformer gate@(T_Comment _ _ _) = identity_transformer gate
 -- 
 -- * /T/ and /T/[sup †].
 -- 
--- It transforms them into the four gates /H/, /S/, /T/, and
--- /CNOT/. Global phases are suppressed. Classical controls and
--- classical gates are not subject to the gate base, and are left
--- untouched.
+-- Global phases are suppressed. Classical controls and classical
+-- gates are not subject to the gate base, and are left untouched.
 -- 
 -- Any gates that are not among the input gates listed above will be
 -- transformed to a semantically correct circuit which may, however,
--- contain gates that are not among the four above output gates. The
--- best way to avoid this is to apply exact and approximate
--- Clifford+/T/ decomposition first.
+-- contain gates that are not in the standard gate set. The best way
+-- to avoid this is to apply exact and approximate Clifford+/T/
+-- decomposition first.
+standard_transformer :: Transformer Circ Qubit Bit
+standard_transformer gate@(T_QGate "H" 1 0 _ _ _) = identity_transformer gate
+standard_transformer gate@(T_QGate "Y" 1 0 _ ncf f) = identity_transformer gate
+standard_transformer gate@(T_QGate "Z" 1 0 _ ncf f) = identity_transformer gate
+standard_transformer gate@(T_QGate "S" 1 0 _ ncf f) = identity_transformer gate
+standard_transformer gate@(T_QGate "T" 1 0 _ ncf f) = identity_transformer gate
+standard_transformer gate@(T_QGate "multinot" _ _ _ _ _) = identity_transformer gate
+standard_transformer gate@(T_QGate "swap" 2 0 _ _ _) = identity_transformer gate
+standard_transformer gate@(T_QGate "W" 2 0 _ _ _) = identity_transformer gate
+
+standard_transformer gate@(T_QGate "not" 1 0 _ ncf f) = f $
+  \[q] [] cs -> without_controls_if ncf $ do
+    with_normalized_controls cs $ \qs -> do
+      case qs of
+        [] -> do
+          gate_X_at q
+        qs -> qnot_at q `controlled` qs
+    return ([q], [], cs)
+
+standard_transformer gate@(T_QGate "X" 1 0 _ ncf f) = f $
+  \[q] [] cs -> without_controls_if ncf $ do
+    with_normalized_controls cs $ \qcs -> do
+      case qcs of
+        [] -> do
+          gate_X_at q
+        qcs -> do
+          qnot_at q `controlled` qcs
+    return ([q],[],cs)
+
+standard_transformer gate@(T_QGate "iX" 1 0 inv ncf f) = f $
+  \[q] [] cs -> without_controls_if ncf $ do
+    with_normalized_controls cs $ \qcs -> do
+      case qcs of
+        [] -> do
+          gate_X_at q
+        qcs -> do
+          reverse_imp_if inv gate_iX_at q `controlled` qcs
+    return ([q],[],cs)
+
+standard_transformer gate@(T_QGate "V" 1 0 inv ncf f) = f $
+  \[q] [] cs -> without_controls_if ncf $ do
+    with_controls cs $ do
+      hadamard_at q
+      reverse_imp_if inv gate_S_inv_at q
+      hadamard_at q
+    return ([q],[],cs)
+
+standard_transformer gate@(T_QGate "E" 1 0 False ncf f) = f $
+  \[q] [] cs -> without_controls_if ncf $ do
+    with_controls cs $ do
+      hadamard_at q
+      gate_S_inv_at q
+    return ([q],[],cs)
+standard_transformer gate@(T_QGate "E" 1 0 True ncf f) = f $
+  \[q] [] cs -> without_controls_if ncf $ do
+    with_controls cs $ do
+      gate_S_at q
+      hadamard_at q
+    return ([q],[],cs)
+
+standard_transformer gate@(T_QGate "YY" 1 0 inv ncf f) = f $
+  \[q] [] cs -> without_controls_if ncf $ do
+    with_controls cs $ do
+      reverse_imp_if inv gate_S_at q
+      hadamard_at q
+      reverse_imp_if inv gate_S_at q
+    return ([q],[],cs)
+
+standard_transformer gate@(T_QGate "omega" 1 0 _ ncf f) = f $
+  \[q] [] cs -> 
+    return ([q],[],cs)
+
+standard_transformer gate@(T_QGate name _ _ inv ncf f) =     
+  error ("standard_transformer: gate not implemented: " ++ show gate)
+
+-- We list catch-all cases explicitly, so that the type-checker can
+-- warn about new gates that must be added to the list.
+standard_transformer gate@(T_QRot _ _ _ _ _ _ _) = identity_transformer gate
+standard_transformer gate@(T_GPhase _ _ _) = identity_transformer gate
+standard_transformer gate@(T_Subroutine _ _ _ _ _ _ _ _ _ _) = identity_transformer gate
+standard_transformer gate@(T_CNot _ _) = identity_transformer gate
+standard_transformer gate@(T_CGate _ _ _) = identity_transformer gate
+standard_transformer gate@(T_CGateInv _ _ _) = identity_transformer gate
+standard_transformer gate@(T_CSwap _ _) = identity_transformer gate
+standard_transformer gate@(T_QPrep _ _) = identity_transformer gate
+standard_transformer gate@(T_QUnprep _ _) = identity_transformer gate
+standard_transformer gate@(T_QInit _ _ _) = identity_transformer gate
+standard_transformer gate@(T_CInit _ _ _) = identity_transformer gate
+standard_transformer gate@(T_QTerm _ _ _) = identity_transformer gate
+standard_transformer gate@(T_CTerm _ _ _) = identity_transformer gate
+standard_transformer gate@(T_QMeas _) = identity_transformer gate
+standard_transformer gate@(T_QDiscard _) = identity_transformer gate
+standard_transformer gate@(T_CDiscard _) = identity_transformer gate
+standard_transformer gate@(T_DTerm _ _) = identity_transformer gate
+standard_transformer gate@(T_Comment _ _ _) = identity_transformer gate
+
+-- ----------------------------------------------------------------------
+-- ** Decomposition into strict gate set
+
+-- | This transformer decomposes a circuit into the strict gate set,
+-- which we define to be:
+-- 
+-- * /H/, /S/, /T/, and /CNOT/.
+-- 
+-- As a precondition, the input circuit must only contain the
+-- following gates:
+-- 
+-- * controlled-not (with one positive or negative control),
+-- 
+-- * any single-qubit Clifford gates,
+-- 
+-- * /T/ and /T/[sup †].
+-- 
+-- Global phases are suppressed. Classical controls and classical
+-- gates are not subject to the gate base, and are left untouched.
+-- 
+-- Any gates that are not among the input gates listed above will be
+-- transformed to a semantically correct circuit which may, however,
+-- contain gates that are not in the strict gate set. The best way to
+-- avoid this is to apply exact and approximate Clifford+/T/
+-- decomposition first.
 strict_transformer :: Transformer Circ Qubit Bit
+strict_transformer gate@(T_QGate "H" 1 0 _ _ _) = identity_transformer gate
+strict_transformer gate@(T_QGate "multinot" _ _ _ _ _) = identity_transformer gate
+strict_transformer gate@(T_QGate "swap" 2 0 _ _ _) = identity_transformer gate
+strict_transformer gate@(T_QGate "W" 2 0 _ _ _) = identity_transformer gate
+
 strict_transformer gate@(T_QGate "not" 1 0 _ ncf f) = f $
   \[q] [] cs -> without_controls_if ncf $ do
     with_normalized_controls_HS cs $ \qs -> do
@@ -556,108 +719,129 @@ strict_transformer gate@(T_QGate "not" 1 0 _ ncf f) = f $
           hadamard_at q
         qs -> qnot_at q `controlled` qs
     return ([q], [], cs)
-strict_transformer gate@(T_QGate "multinot" _ _ _ _ _) = identity_transformer gate
-strict_transformer gate@(T_QGate "H" 1 0 _ _ _) = identity_transformer gate
-strict_transformer gate@(T_QGate "swap" 2 0 _ _ _) = identity_transformer gate
-strict_transformer gate@(T_QGate "W" 2 0 _ _ _) = identity_transformer gate
-strict_transformer gate@(T_QGate name _ _ inv ncf f) = f $
-  \qs gctls cs -> without_controls_if ncf $ do
+
+strict_transformer gate@(T_QGate "X" 1 0 _ ncf f) = f $
+  \[q] [] cs -> without_controls_if ncf $ do
     with_normalized_controls_HS cs $ \qcs -> do
-      case (name, qs, gctls, inv) of
-        
-        ("X", [q], [], _) -> do
-          case qcs of
-            [] -> do
-              hadamard_at q
-              gate_S_at q
-              gate_S_at q
-              hadamard_at q
-            qcs -> do
-              qnot_at q `controlled` qcs
-        
-        ("Y", [q], [], _) -> do
-          with_controls qcs $ do
-            gate_S_at q
-            hadamard_at q
-            gate_S_at q
-            gate_S_at q
-            hadamard_at q
-            gate_S_at q
-            gate_S_at q
-            gate_S_at q
-          
-        ("Z", [q], [], _) -> do
-          with_controls qcs $ do
-            gate_S_at q
-            gate_S_at q
-          
-        ("iX", [q], [], _) -> do
-          case qcs of
-            [] -> do
-              hadamard_at q
-              gate_S_at q
-              gate_S_at q
-              hadamard_at q
-            qcs -> do
-              reverse_imp_if inv gate_iX_at q `controlled` qcs
-        
-        ("S", [q], [], False) -> do
-          with_controls qcs $ do
-            gate_S_at q
-          
-        ("S", [q], [], True) -> do
-          with_controls qcs $ do
-            gate_S_at q
-            gate_S_at q
-            gate_S_at q
-          
-        ("T", [q], [], False) -> do
-          with_controls qcs $ do
-            gate_T_at q
-          
-        ("T", [q], [], True) -> do
-          with_controls qcs $ do
-            gate_T_at q
-            gate_S_at q
-            gate_S_at q
-            gate_S_at q
-          
-        ("V", [q], [], False) -> do
-          with_controls qcs $ do
-            hadamard_at q
-            gate_S_at q
-            gate_S_at q
-            gate_S_at q
-            hadamard_at q
-          
-        ("V", [q], [], True) -> do
-          with_controls qcs $ do
-            hadamard_at q
-            gate_S_at q
-            hadamard_at q
-          
-        ("E", [q], [], False) -> do
-          case qcs of
-            [] -> do
-              hadamard_at q
-              gate_S_at q
-              gate_S_at q
-              gate_S_at q
-            qcs -> do
-              gate_E_at q `controlled` qcs
-          
-        ("E", [q], [], True) -> do
-          case qcs of
-            [] -> do
-              gate_S_at q
-              hadamard_at q
-            qcs -> do
-              gate_E_at q `controlled` qcs
-          
-        ("omega", [q], [], _) -> return ()
-            
-        _ -> error ("strict_transformer: gate not implemented: " ++ show gate ++ "###" ++ show (name,qs,gctls))
-    return (qs,gctls,cs)
+      case qcs of
+        [] -> do
+          hadamard_at q
+          gate_S_at q
+          gate_S_at q
+          hadamard_at q
+        qcs -> do
+          qnot_at q `controlled` qcs
+    return ([q],[],cs)
+
+strict_transformer gate@(T_QGate "Y" 1 0 _ ncf f) = f $
+  \[q] [] cs -> without_controls_if ncf $ do
+    with_controls cs $ do
+      gate_S_at q
+      hadamard_at q
+      gate_S_at q
+      gate_S_at q
+      hadamard_at q
+      gate_S_at q
+      gate_S_at q
+      gate_S_at q
+    return ([q],[],cs)
+
+strict_transformer gate@(T_QGate "Z" 1 0 _ ncf f) = f $
+  \[q] [] cs -> without_controls_if ncf $ do
+    with_controls cs $ do
+      gate_S_at q
+      gate_S_at q
+    return ([q],[],cs)
+
+strict_transformer gate@(T_QGate "iX" 1 0 inv ncf f) = f $
+  \[q] [] cs -> without_controls_if ncf $ do
+    with_normalized_controls_HS cs $ \qcs -> do
+      case qcs of
+        [] -> do
+          hadamard_at q
+          gate_S_at q
+          gate_S_at q
+          hadamard_at q
+        qcs -> do
+          reverse_imp_if inv gate_iX_at q `controlled` qcs
+    return ([q],[],cs)
+
+strict_transformer gate@(T_QGate "S" 1 0 False ncf f) = 
+  identity_transformer gate
+strict_transformer gate@(T_QGate "S" 1 0 True ncf f) = f $
+  \[q] [] cs -> without_controls_if ncf $ do
+    with_controls cs $ do
+      gate_S_at q
+      gate_S_at q
+      gate_S_at q
+    return ([q],[],cs)
+
+strict_transformer gate@(T_QGate "T" 1 0 False ncf f) = f $
+  \[q] [] cs -> without_controls_if ncf $ do
+    with_controls cs $ do
+      gate_T_at q
+    return ([q],[],cs)
+strict_transformer gate@(T_QGate "T" 1 0 True ncf f) = f $
+  \[q] [] cs -> without_controls_if ncf $ do
+    with_controls cs $ do
+      gate_T_at q
+      gate_S_at q
+      gate_S_at q
+      gate_S_at q
+    return ([q],[],cs)
+
+strict_transformer gate@(T_QGate "V" 1 0 False ncf f) = f $
+  \[q] [] cs -> without_controls_if ncf $ do
+    with_controls cs $ do
+      hadamard_at q
+      gate_S_at q
+      gate_S_at q
+      gate_S_at q
+      hadamard_at q
+    return ([q],[],cs)
+strict_transformer gate@(T_QGate "V" 1 0 True ncf f) = f $
+  \[q] [] cs -> without_controls_if ncf $ do
+    with_controls cs $ do
+      hadamard_at q
+      gate_S_at q
+      hadamard_at q
+    return ([q],[],cs)
+
+strict_transformer gate@(T_QGate "E" 1 0 False ncf f) = f $
+  \[q] [] cs -> without_controls_if ncf $ do
+    with_controls cs $ do
+      hadamard_at q
+      gate_S_at q
+      gate_S_at q
+      gate_S_at q
+    return ([q],[],cs)
+strict_transformer gate@(T_QGate "E" 1 0 True ncf f) = f $
+  \[q] [] cs -> without_controls_if ncf $ do
+    with_controls cs $ do
+      gate_S_at q
+      hadamard_at q
+    return ([q],[],cs)
+
+strict_transformer gate@(T_QGate "YY" 1 0 inv ncf f) = f $
+  \[q] [] cs -> without_controls_if ncf $ do
+    with_controls cs $ do
+      when (inv) $ do
+        gate_S_at q
+        gate_S_at q
+      gate_S_at q
+      hadamard_at q
+      gate_S_at q
+      when (inv) $ do
+        gate_S_at q
+        gate_S_at q
+    return ([q],[],cs)
+
+strict_transformer gate@(T_QGate "omega" 1 0 _ ncf f) = f $
+  \[q] [] cs -> 
+    return ([q],[],cs)
+
+strict_transformer gate@(T_QGate name _ _ inv ncf f) =     
+  error ("strict_transformer: gate not implemented: " ++ show gate)
 
 -- We list catch-all cases explicitly, so that the type-checker can
 -- warn about new gates that must be added to the list.
@@ -708,23 +892,23 @@ trimcontrols_unary :: (QCData qa, QCData qb) => (qa -> Circ qb) -> (qa -> Circ q
 trimcontrols_unary = transform_unary_dynamic trimcontrols_dtransformer
 
 -- | Handle subroutines for the 'approx_ct_transformer'.
-approx_ct_subroutine :: (RandomGen g) => Precision -> g -> BoxId -> TypedSubroutine -> Circ ()
-approx_ct_subroutine prec g boxid sub@(TypedSubroutine ocirc is os ctl) = do
+approx_ct_subroutine :: (RandomGen g) => KeepPhase -> Precision -> g -> BoxId -> TypedSubroutine -> Circ ()
+approx_ct_subroutine kp prec g boxid sub@(TypedSubroutine ocirc is os ctl) = do
   let circ = open_subroutine sub
-  let circ' = approx_ct_unary prec g circ
+  let circ' = approx_ct_unary kp prec g circ
   let OCircuit (win, (arity,_,_,_), _) = ocirc
   let ein = endpoints_of_wires_in_arity arity win
   provide_subroutine_generic (\x -> "approx_ct_subroutine: " ++ x) boxid False circ' ein
 
 -- | Dynamic transformer for the 'approx_ct_transformer'.
-approx_ct_dtransformer :: (RandomGen g) => Precision -> g -> DynamicTransformer Circ Qubit Bit
-approx_ct_dtransformer prec g = identity_dynamic_transformer {
-  transformer = approx_ct_transformer prec g,
-  define_subroutine = approx_ct_subroutine prec g}
+approx_ct_dtransformer :: (RandomGen g) => KeepPhase -> Precision -> g -> DynamicTransformer Circ Qubit Bit
+approx_ct_dtransformer kp prec g = identity_dynamic_transformer {
+  transformer = approx_ct_transformer kp prec g,
+  define_subroutine = approx_ct_subroutine kp prec g}
 
 -- | Unary transformer for the 'approx_ct_transformer'.
-approx_ct_unary :: (RandomGen g, QCData qa, QCData qb) => Precision -> g -> (qa -> Circ qb) -> (qa -> Circ qb)
-approx_ct_unary prec g = transform_unary_dynamic (approx_ct_dtransformer prec g)
+approx_ct_unary :: (RandomGen g, QCData qa, QCData qb) => KeepPhase -> Precision -> g -> (qa -> Circ qb) -> (qa -> Circ qb)
+approx_ct_unary kp prec g = transform_unary_dynamic (approx_ct_dtransformer kp prec g)
 
 -- | Handle subroutines for the 'exact_ct_transformer'.
 exact_ct_subroutine :: BoxId -> TypedSubroutine -> Circ ()
@@ -745,6 +929,25 @@ exact_ct_dtransformer = identity_dynamic_transformer {
 exact_ct_unary :: (QCData qa, QCData qb) => (qa -> Circ qb) -> (qa -> Circ qb)
 exact_ct_unary = transform_unary_dynamic exact_ct_dtransformer
 
+-- | Handle subroutines for the 'standard_transformer'.
+standard_subroutine :: BoxId -> TypedSubroutine -> Circ ()
+standard_subroutine boxid sub@(TypedSubroutine ocirc is os ctl) = do
+  let circ = open_subroutine sub
+  let circ' = standard_unary circ
+  let OCircuit (win, (arity,_,_,_), _) = ocirc
+  let ein = endpoints_of_wires_in_arity arity win
+  provide_subroutine_generic (\x -> "standard_subroutine: " ++ x) boxid False circ' ein
+
+-- | Dynamic transformer for the 'standard_transformer'.
+standard_dtransformer :: DynamicTransformer Circ Qubit Bit
+standard_dtransformer = identity_dynamic_transformer {
+  transformer = standard_transformer,
+  define_subroutine = standard_subroutine}
+
+-- | Unary transformer for the 'standard_transformer'.
+standard_unary :: (QCData qa, QCData qb) => (qa -> Circ qb) -> (qa -> Circ qb)
+standard_unary = transform_unary_dynamic standard_dtransformer
+
 -- | Handle subroutines for the 'strict_transformer'.
 strict_subroutine :: BoxId -> TypedSubroutine -> Circ ()
 strict_subroutine boxid sub@(TypedSubroutine ocirc is os ctl) = do
@@ -786,10 +989,10 @@ trimcontrols_generic f = h where
 -- <http://arxiv.org/abs/1212.6253>.  It requires a precision
 -- parameter, as well as a source of randomness. Other gates
 -- are unchanged.
-approx_ct_generic :: (RandomGen g, QCData qa, QCData qb, QCurry qfun qa qb) => Precision -> g -> qfun -> qfun
-approx_ct_generic prec g f = h where
+approx_ct_generic :: (RandomGen g, QCData qa, QCData qb, QCurry qfun qa qb) => KeepPhase -> Precision -> g -> qfun -> qfun
+approx_ct_generic kp prec g f = h where
   f1 = quncurry f
-  h1 = approx_ct_unary prec g f1
+  h1 = approx_ct_unary kp prec g f1
   h = qcurry h1
 
 -- | This transformer decomposes all gates that permit exact
@@ -813,7 +1016,13 @@ exact_ct_generic f = h where
   h1 = exact_ct_unary f1
   h = qcurry h1
 
--- | This transformer accepts the following gates:
+-- | This transformer decomposes a circuit into the standard gate set,
+-- which we define to be:
+-- 
+-- * /X/, /Y/, /Z/, /H/, /S/, /S/[sup †], /T/, /T/[sup †], and /CNOT/.
+-- 
+-- As a precondition, the input circuit must only contain the
+-- following gates:
 -- 
 -- * controlled-not (with one positive or negative control),
 -- 
@@ -821,16 +1030,42 @@ exact_ct_generic f = h where
 -- 
 -- * /T/ and /T/[sup †].
 -- 
--- It decomposes them into the four gates /H/, /S/, /T/, and /CNOT/
--- (with a single positive control). Global phases are
--- suppressed. Classical controls and classical gates are not subject
--- to the gate base, and are left untouched.
+-- Global phases are suppressed. Classical controls and classical
+-- gates are not subject to the gate base, and are left untouched.
 -- 
 -- Any gates that are not among the input gates listed above will be
 -- transformed to a semantically correct circuit which may, however,
--- contain gates that are not among /H/, /S/, /T/, and /CNOT/. The
--- best way to avoid this is to apply both the exact and approximate
--- Clifford+/T/ decomposer first.
+-- contain gates that are not in the standard gate set. The best way
+-- to avoid this is to apply exact and approximate Clifford+/T/
+-- decomposition first.
+standard_generic :: (QCData qa, QCData qb, QCurry qfun qa qb) => qfun -> qfun
+standard_generic f = h where
+  f1 = quncurry f
+  h1 = standard_unary f1
+  h = qcurry h1
+
+-- | This transformer decomposes a circuit into the strict gate set,
+-- which we define to be:
+-- 
+-- * /H/, /S/, /T/, and /CNOT/.
+-- 
+-- As a precondition, the input circuit must only contain the
+-- following gates:
+-- 
+-- * controlled-not (with one positive or negative control),
+-- 
+-- * any single-qubit Clifford gates,
+-- 
+-- * /T/ and /T/[sup †].
+-- 
+-- Global phases are suppressed. Classical controls and classical
+-- gates are not subject to the gate base, and are left untouched.
+-- 
+-- Any gates that are not among the input gates listed above will be
+-- transformed to a semantically correct circuit which may, however,
+-- contain gates that are not in the strict gate set. The best way to
+-- avoid this is to apply exact and approximate Clifford+/T/
+-- decomposition first.
 strict_generic :: (QCData qa, QCData qb, QCurry qfun qa qb) => qfun -> qfun
 strict_generic f = h where
   f1 = quncurry f
