@@ -21,12 +21,25 @@
 -- Cambridge University Press, 2002.
 -- 
 -- * M. Amy, D. Maslov, M. Mosca, and M. Roetteler,
--- /A meet-in-the-middle algorithm for fast synthesis/ 
--- /of depth-optimal quantum circuits/,
--- <http://arxiv.org/abs/1206.0758>.
+-- A meet-in-the-middle algorithm for fast synthesis 
+-- of depth-optimal quantum circuits,
+-- /IEEE Transactions on Computer-Aided Design of/ 
+-- /Integrated Circuits and Systems/ 32(6):818-830.
+-- Also available from <http://arxiv.org/abs/1206.0758>.
 -- 
--- * P. Selinger, /Quantum circuits of T-depth one/,
--- <http://arxiv.org/abs/1210.0974>.
+-- * A. Barenco, C. H. Bennett, R. Cleve, D. P. DiVincenzo,
+-- N. Margolus, P. Shor, T. Sleator, J. A. Smolin, and H. Weinfurter,
+-- Elementary gates for quantum computation,
+-- /Physical Review A/ 52(5):3457-3467, 1995. 
+-- Also available from <http://arxiv.org/abs/quantph/9503016>.
+-- 
+-- * P. Selinger, Quantum circuits of T-depth one,
+-- /Physical Review A/ 87, 042302 (4 pages), 2013.
+-- Also available from <http://arxiv.org/abs/1210.0974>.
+-- 
+-- * B. Giles and P. Selinger, Exact synthesis of multiqubit
+-- Clifford+T circuits, /Physical Review A/ 87, 032332 (7 pages),
+-- 2013.  Also available from <http://arxiv.org/abs/1212.0506>.
 
 module QuipperLib.GateDecompositions where
 
@@ -102,6 +115,22 @@ toffoli_V_at q c1 c2 = do
   gate_V_at q `controlled` c2
   comment_with_label "EXIT: toffoli_V" (q,c1,c2) ("q","c1","c2")
   
+-- | Decomposition of the Toffoli gate into the Clifford+/T/ base,
+-- using /T/-depth 1 and four ancillas. From
+-- <http://arxiv.org/abs/1210.0974> (Figure 1). The first argument is
+-- the target, and the other two are the controls. The controls can be
+-- positive or negative.
+-- 
+-- \[image toffoli_S.png]
+toffoli_S_at :: Qubit -> Signed Qubit -> Signed Qubit -> Circ ()
+toffoli_S_at q c1 c2 = do
+  comment_with_label "ENTER: toffoli_S" (q,c1,c2) ("q","c1","c2")
+  without_comments $ do
+    hadamard_at q
+    ccZ_S_at q c1 c2
+    hadamard_at q
+  comment_with_label "EXIT: toffoli_S" (q,c1,c2) ("q","c1","c2")
+  
 -- | Decomposition of the doubly-controlled /iX/-gate into the
 -- Clifford+/T/ base, using /T/-count 4 and /T/-depth 2. Adapted from
 -- (<http://arxiv.org/abs/1210.0974>, Figure 10). The first argument
@@ -155,6 +184,43 @@ cc_iX_simple_at q c1 c2 = do
   hadamard_at q
   comment_with_label "EXIT: cc_iX_simple" (q,c1,c2) ("q","c1","c2")
   
+-- | Decomposition of the doubly-controlled /iX/-gate into the
+-- Clifford+/T/ base, using /T/-depth 1 and one ancilla. Adapted from
+-- (<http://arxiv.org/abs/1210.0974>, Figure 9). The first argument
+-- is the target, and the other two are the controls. The controls can
+-- be positive or negative.
+-- 
+-- \[image cc_iX_S.png]
+cc_iX_S_at :: Qubit -> Signed Qubit -> Signed Qubit -> Circ ()
+cc_iX_S_at q c1 c2 = do
+  comment_with_label "ENTER: cc_iX_S" (q,c1,c2) ("q","c1","c2")
+  let sx = get_sign c1
+      sy = get_sign c2
+      x = from_signed c1
+      y = from_signed c2
+      z = q
+  hadamard_at z
+  with_ancilla $ \w -> do
+    qnot y `controlled` z
+    qnot w `controlled` x
+    qnot x `controlled` z
+    qnot w `controlled` y
+    reverse_imp_if (not sx) gate_T_at x
+    reverse_imp_if (not sy) gate_T_at y
+    gate_T_inv_at z
+    reverse_imp_if (sx `xor` sy) gate_T_inv_at w
+    qnot w `controlled` y
+    qnot x `controlled` z
+    qnot w `controlled` x
+    qnot y `controlled` z
+  when (not sx && not sy) $ do  -- need a phase correction
+    gate_omega_at x
+    gate_omega_at y
+  hadamard_at z
+  comment_with_label "EXIT: cc_iX_S" (q,c1,c2) ("q","c1","c2")
+  where
+    xor = (/=)
+    
 -- | Decomposition of the doubly-controlled /Z/-gate into the
 -- Clifford+/T/ base. Adapted from Amy et
 -- al. (<http://arxiv.org/abs/1206.0758v3>, Figure 13). The controls can
@@ -186,6 +252,48 @@ ccZ_AMMR_at q c1 c2 = do
   where
     xor = (/=)
 
+-- | Decomposition of the doubly-controlled /Z/-gate into the
+-- Clifford+/T/ base, using /T/-depth 1 and four ancillas. From
+-- Selinger (<http://arxiv.org/abs/1210.0974>, Figure 1). The controls
+-- can be positive or negative.
+-- 
+-- \[image ccZ_S.png]
+ccZ_S_at :: Qubit -> Signed Qubit -> Signed Qubit -> Circ ()
+ccZ_S_at q c1 c2 = do
+  comment_with_label "ENTER: ccZ_S" (q,c1,c2) ("q","c1","c2")
+  let sx = get_sign c1
+      sy = get_sign c2
+      x = from_signed c1
+      y = from_signed c2
+      z = q
+  with_ancilla_init (0,0,0,0) $ \(xyz, xy, yz, xz) -> do
+    qnot yz `controlled` y
+    qnot xyz `controlled` x
+    qnot xy `controlled` y
+    qnot yz `controlled` z
+    qnot xz `controlled` xyz
+    qnot xy `controlled` x
+    qnot xz `controlled` z
+    qnot xyz `controlled` yz
+    reverse_imp_if (not sx) gate_T_at x
+    reverse_imp_if (not sy) gate_T_at y
+    gate_T_at z
+    reverse_imp_if (sx `xor` sy) gate_T_at xyz
+    reverse_imp_if (sx `xor` sy) gate_T_inv_at xy
+    reverse_imp_if (not sy) gate_T_inv_at yz
+    reverse_imp_if (not sx) gate_T_inv_at xz
+    qnot xyz `controlled` yz
+    qnot xz `controlled` z
+    qnot xy `controlled` x
+    qnot xz `controlled` xyz
+    qnot yz `controlled` z
+    qnot xy `controlled` y
+    qnot xyz `controlled` x
+    qnot yz `controlled` y
+  comment_with_label "EXIT: ccZ_S" (q,c1,c2) ("q","c1","c2")
+  where
+    xor = (/=)
+
 -- | Decomposition of the Fredkin (controlled-Swap) gate into the
 -- Clifford+/T/ base. The first two arguments are the targets, and the
 -- last one the control. The controls can be positive or negative.
@@ -322,11 +430,49 @@ controlled_V_at q c = do
   comment_with_label "EXIT: controlled_V" (q,c) ("q","c")
 
 -- | Decomposition of a controlled /E/-gate into the Clifford+/T/
--- base. Currently not implemented; this function exists here as a
--- placeholder.
+-- base. The first argument is the target, and the second one is the
+-- control. The control can be positive or negative.
+-- 
+-- \[image controlled_E.png]
 controlled_E_at :: Qubit -> Signed Qubit -> Circ ()
 controlled_E_at q c = do
-  error "controlled_E_at: not implemented"
+  comment_with_label "ENTER: controlled_E" (q,c) ("q","c")
+  with_signed_qubit c $ \r -> do
+    gate_H_at q
+    gate_S_at r
+    gate_T_at q
+    qnot_at q `controlled` r
+    gate_T_inv_at q
+    gate_H_at q
+    qnot_at r `controlled` q
+    gate_T_at r
+    gate_T_inv_at q
+    qnot_at r `controlled` q
+  comment_with_label "EXIT: controlled_E" (q,c) ("q","c")
+
+-- | Decomposition of a controlled [bold Y]-gate into the Clifford+/T/
+-- base. The gate is from the Ground State Estimation algorithm and is
+-- defined as [bold Y] = /SHS/, or equivalently,
+-- 
+-- \[image Y.png]
+-- 
+-- It should not be confused with the Pauli /Y/ gate.
+-- The first argument is the target, and the second one is the
+-- control. The control can be positive or negative.
+-- 
+-- \[image controlled_YY.png]
+controlled_YY_at :: Qubit -> Signed Qubit -> Circ ()
+controlled_YY_at q c = do
+  comment_with_label "ENTER: controlled_YY" (q,c) ("q","c")
+  gate_S_at q
+  qnot_at q `controlled` c
+  gate_S_inv_at q
+  hadamard_at q
+  gate_T_inv_at q
+  qnot_at q `controlled` c
+  gate_T_at q
+  hadamard_at q
+  comment_with_label "EXIT: controlled_YY" (q,c) ("q","c")
 
 -- | A \"plain\" Toffoli gate, not decomposed. This is provided for
 -- convenience, for example to use with 'with_combined_controls'.
@@ -341,6 +487,91 @@ cc_iX_plain_at :: Qubit -> Signed Qubit -> Signed Qubit -> Circ ()
 cc_iX_plain_at q c1 c2 = do
   gate_iX_at q `controlled` (c1,c2)
 
+-- | Decomposition of an /m/-times controlled not-gate, using /m/−2
+-- ancillas that do not need be initialized in a particular
+-- state. Adapted from Barenco et al.
+-- (<http://arxiv.org/abs/quantph/9503016>, Lemma 7.2).
+-- 
+-- In addition to what is shown in Barenco et al., this function
+-- permits some Toffoli gates to be replaced by doubly-controlled
+-- /iX/-gates. This may be beneficial in gate bases, such as
+-- Clifford+/T/, where a doubly-controlled /iX/-gate has a simpler
+-- representation than a Toffoli gate.
+-- 
+-- The first argument is a Toffoli gate to use in the
+-- decomposition. The second argument may be either a Toffoli gate or
+-- a doubly-controlled /iX/ gate. The third argument is the target,
+-- the fourth argument is a list of qubits to be used as ancillas, and
+-- the fifth argument is a list of signed controls. The ancillas need
+-- not be initialized, and are returned in their original state.
+-- 
+-- The size of this circuit is linear in the number of controls; the
+-- decomposition uses 4/m/−8 doubly-controlled gates for /m/ ≥ 3.
+-- 
+-- \[image multi_cnot_barenco.png]
+multi_cnot_barenco_at :: (Qubit -> Signed Qubit -> Signed Qubit -> Circ ()) -> (Qubit -> Signed Qubit -> Signed Qubit -> Circ ()) -> Qubit -> [Qubit] -> [Signed Qubit] -> Circ ()
+multi_cnot_barenco_at my_toffoli_at my_ciX_at q as cs =
+  case cs of
+    [] -> do
+      qnot_at q
+    [c] -> do
+      qnot_at q `controlled` c
+    [c1,c2] -> do
+      my_toffoli_at q c1 c2
+    c:cs -> do
+      case as of
+        [] -> error "multi_cnot_barenco_at: too few ancillas"
+        a:as -> do
+          my_toffoli_at q (Signed a True) c
+          aux cs (a:as)
+          my_toffoli_at q (Signed a True) c
+          reverse_generic_imp aux cs (a:as)
+  where
+    aux :: [Signed Qubit] -> [Qubit] -> Circ ()
+    aux [] as = return ()
+    aux [c] as = return ()
+    aux [c1,c2] (a:as) = do
+      my_ciX_at a c1 c2
+    aux (c:cs) (a1:a2:as) = do
+      my_ciX_at a1 (Signed a2 True) c
+      aux cs (a2:as)
+      my_ciX_at a1 (Signed a2 True) c
+    aux _ _ = error "multi_cnot_barenco_at: too few ancillas"
+
+-- | Decomposition of a multiply-controlled /iX/-gate, using no
+-- ancillas. Adapted from Giles and Selinger
+-- (<http://arxiv.org/abs/1212.0506>, Section 5.2).
+-- 
+-- The first argument is a Toffoli gate or a doubly-controlled
+-- /iX/-gate. The third argument is the target, and the fourth
+-- argument is a list of signed controls.
+-- 
+-- The size of this circuit is linear in the number of controls; the
+-- decomposition uses 8/m/−32 doubly-controlled gates, 4 /T/-gates,
+-- and 2 /H/-gates, for /m/ ≥ 6.
+-- 
+-- \[image multi_ciX_noancilla.png]
+multi_ciX_noancilla_at :: (Qubit -> Signed Qubit -> Signed Qubit -> Circ ()) -> Qubit -> [Signed Qubit] -> Circ ()
+multi_ciX_noancilla_at my_ciX_at q [] = gate_iX_at q
+multi_ciX_noancilla_at my_ciX_at q [c] = gate_iX_at q `controlled` c
+multi_ciX_noancilla_at my_ciX_at q [c1,c2] = gate_iX_at q `controlled` [c1,c2]
+multi_ciX_noancilla_at my_ciX_at q cs = do
+  hadamard_at q
+  gate_T_inv_at q
+  multi_cnot_barenco_at my_ciX_at my_ciX_at q as1 cs2
+  gate_T_at q
+  multi_cnot_barenco_at my_ciX_at my_ciX_at q as2 cs1
+  gate_T_inv_at q
+  reverse_generic_imp (multi_cnot_barenco_at my_ciX_at my_ciX_at) q as1 cs2
+  gate_T_at q
+  reverse_generic_imp (multi_cnot_barenco_at my_ciX_at my_ciX_at) q as2 cs1
+  hadamard_at q
+  where
+    n = length cs
+    (cs1, cs2) = splitAt (n `div` 2) cs
+    as1 = map from_signed cs1
+    as2 = map from_signed cs2
+    
 -- ----------------------------------------------------------------------
 -- * Decomposition of controls
 
@@ -350,6 +581,23 @@ partition_controls cs = (qcs, ccs) where
   qcs = [ Signed q b | Signed (Endpoint_Qubit q) b <- cs ]
   ccs = [ Signed c b | Signed (Endpoint_Bit c) b <- cs ]
 
+-- | Given a function that expects a qubit (typically as a control),
+-- turn it into a function that can handle a /signed/ (positive or
+-- negative) qubit. This is done by conjugating the circuit with
+-- negations on both sides, if the sign is negative. Usage:
+-- 
+-- > with_signed_qubit c $ \q -> do
+-- >   <<<code using q>>>
+-- 
+-- \[image with_signed_qubit.png]
+with_signed_qubit :: Signed Qubit -> (Qubit -> Circ b) -> Circ b
+with_signed_qubit (Signed q True) f = f q
+with_signed_qubit (Signed q False) f = do
+  gate_X_at q
+  b <- f q
+  gate_X_at q
+  return b
+
 -- | Decompose quantum controls recursively until at most /n/ remain,
 -- and then pass these reduced controls to the given circuit.
 -- Precondition: /n/ ≥ 1.  
