@@ -140,7 +140,7 @@ module Quipper.Generic (
   nbox,
   box_loopM,
   loopM_boxed_if,
-  unbox
+  inline_subroutine
   ) where
 
 -- import other Quipper stuff
@@ -803,7 +803,7 @@ qubits_of_qdata qa = qdata_sequentialize (shapetype_q qa) qa
 -- A \"length mismatch\" error occurs if the list does not have
 -- exactly the required length.
 qdata_of_qubits :: (QData qa) => qa -> [Qubit] -> qa
-qdata_of_qubits qa list = qdata_unsequentialize (shapetype_q qa) list
+qdata_of_qubits qa list = qdata_unsequentialize qa list
 
 -- | Return the list of endpoints that form the leaves of the given
 -- 'QCData'. The leaves are ordered in some fixed, but arbitrary
@@ -1621,40 +1621,3 @@ inline_subroutine name ns inputs = do
       out_bind <- apply_circuit_with_bindings circuit in_bind
       let outputs = unbind_list out_bind wout
       return outputs
-
-
--- | A transformer to peel away one level of boxing. Transforms any
--- top-level subroutine gate into its corresponding circuit.
-unbox_transformer :: Transformer Circ Qubit Bit
-unbox_transformer (T_Subroutine name inv ncf _ _ _ ws2 a2 (RepeatFlag reps) f) = f $
-  \namespace ws c -> do
-    outputs <- loopM reps ws
-      ((without_controls_if ncf) .
-       (with_controls c) .
-       ((if inv then flip reverse_generic (endpoints_of_wires_in_arity a2 ws2) else id)
-        (inline_subroutine name namespace)))
-    return (outputs, c)
-unbox_transformer x = identity_transformer x
-
-
--- | Peel away one level of boxing from a circuit. Transforms any
--- top-level subroutine gate into its corresponding circuit.
-unbox_unary :: (QCData x, QCData y) => (x -> Circ y) -> (x -> Circ y)
-unbox_unary circ = transform_unary unbox_transformer circ 
-
--- | Peel away one level of boxing from a circuit. Transforms any
--- top-level subroutine gate into its corresponding circuit.
---   
--- The type of this heavily overloaded function is difficult to
--- read. In more readable form, it has all of the following types:
--- 
--- > unbox :: (QCData x) =>  Circ x -> Circ x
--- > unbox :: (QCData x, QCData y) =>  (x -> Circ y) -> (x -> Circ y)
--- > unbox :: (QCData x, QCData y, QCData z) => (x -> y -> Circ z) -> (x -> y -> Circ z)
--- 
--- and so forth.
-unbox :: (QCData x, QCData y, QCurry qfun x y) => qfun -> qfun
-unbox f = g where
-  f1 = quncurry f
-  g1 = unbox_unary f1
-  g = qcurry g1
