@@ -172,7 +172,6 @@ module Quipper.Monad (
   controlled,
   without_controls,
   without_controls_if,
-  with_signed_qubit,  
   
   -- ** Deprecated special cases
   qinit_qubit_ancilla,
@@ -775,7 +774,7 @@ rGate n q = do
 -- denotes the \"right\" qubit.
 gate_W :: Qubit -> Qubit -> Circ (Qubit, Qubit)
 gate_W q1 q2 = do
-  named_gate_qulist "W" False [q1, q1] []
+  named_gate_qulist "W" False [q1, q2] []
   return (q1, q2)
 
 -- | Apply an /iX/ gate. This gate is used similarly to the Pauli /X/
@@ -1550,21 +1549,6 @@ without_controls_if :: NoControlFlag -> Circ a -> Circ a
 without_controls_if True = without_controls
 without_controls_if False = id
 
--- | Given a function that expects a qubit (typically as a control),
--- turn it into a function that can handle a /signed/ (positive or
--- negative) qubit. This is done by conjugating the circuit with
--- negations on both sides, if the sign is negative. Usage:
--- 
--- > with_signed_qubit c $ \q -> do
--- >   <<<code using q>>>
-with_signed_qubit :: Signed Qubit -> (Qubit -> Circ b) -> Circ b
-with_signed_qubit (Signed q True) f = f q
-with_signed_qubit (Signed q False) f = do
-  gate_X_at q
-  b <- f q
-  gate_X_at q
-  return b
-
 -- ----------------------------------------------------------------------
 -- ** Deprecated special cases of without_controls
 
