@@ -276,8 +276,8 @@ ascii_of_bcircuit (c,s) =
 ascii_of_subroutine :: (BoxId, TypedSubroutine) -> String
 ascii_of_subroutine (boxid, TypedSubroutine ocirc input_strux output_strux ctrble) =
   "\n" 
-  ++ "Subroutine: " ++ name ++ "\n"
-  ++ "Shape: " ++ shape ++ "\n"
+  ++ "Subroutine: " ++ show name ++ "\n"
+  ++ "Shape: " ++ show shape ++ "\n"
   ++ "Controllable: " ++ (case ctrble of {AllCtl -> "yes"; NoCtl -> "no"; OnlyClassicalCtl -> "classically"}) ++ "\n"
   ++ ascii_of_ocircuit ocirc
     where
@@ -501,9 +501,15 @@ assign_x_coordinates :: FormatStyle -> [Gate] -> X -> (X, [(Gate, X)])
 assign_x_coordinates fs gs x0 =
   let ((x,ws), xgs) = mapAccumL (\ (x, ws) g ->
 	-- count the wires attached to the gate. If there is precisely
-	-- one (unary gate), merge it with adjacent unary gates.
-	case wirelist_of_gate g of
-          [w] ->
+	-- one (unary gate), merge it with adjacent unary gates. Do
+	-- not merge comments.
+        let merge = case (g, wirelist_of_gate g) of
+              (Comment _ _ _, _) -> Nothing
+              (_, [w]) -> Just w
+              (_, _) -> Nothing
+        in
+	case merge of
+          Just w ->
             if not (w `elem` ws) then
 	      ((x, w:ws), (g, x))
 	    else
